import time
import unittest

import rpyc
from rpyc.utils.registry import TCPRegistryServer, TCPRegistryClient
from rpyc.utils.registry import UDPRegistryServer, UDPRegistryClient


PRUNING_TIMEOUT = 5


class BaseRegistryTest(object):
    def _get_server(self):
        raise NotImplementedError

    def _get_client(self):
        raise NotImplementedError

    def setUp(self):
        self.server = self._get_server()
        self.server.logger.quiet = True
        self.server_thread = rpyc.spawn(self.server.start)

    def tearDown(self):
        self.server.close()
        self.server_thread.join()

    def test_api(self):
        c = self._get_client()
        c.logger.quiet = True
        c.register(("FOO",), 12345)
        c.register(("FOO",), 45678)
        res = c.discover("FOO")
        expected = (12345, 45678)
        self.assertEqual(set(p for _, p in res), set(expected))
        c.unregister(12345)
        res = c.discover("FOO")
        expected = (45678,)
        self.assertEqual(set(p for _, p in res), set(expected))
        res = c.list()
        expected = ("FOO",)
        self.assertEqual(set(p for p in res), set(expected))
        c.register(("BAR",), 54321)
        res = c.list()
        expected = ("FOO", "BAR")
        self.assertEqual(set(res), set(expected))

    def test_pruning(self):
        c = self._get_client()
        c.logger.quiet = True
        c.register(("BAR",), 17171)

        time.sleep(1)
        res = c.discover("BAR")
        self.assertEqual(set(p for _, p in res), set((17171,)))

        time.sleep(PRUNING_TIMEOUT)
        res = c.discover("BAR")
        self.assertEqual(res, ())

    def test_listing(self):
        c = self._get_client()
        c.logger.quiet = True

        c.register(("FOO",), 12345)
        c.register(("BAR", ), 54321, interface='127.0.0.2')
        host_ip = c.discover("FOO")[0][0]

        # test basic listing
        res = c.list()
        expected = ("FOO", "BAR")
        self.assertEqual(set(p for p in res), set(expected))

        # test listing with filter
        res = c.list(filter_host=host_ip)
        expected = ("FOO",)
        self.assertEqual(set(res), set(expected))


class TestTcpRegistry(BaseRegistryTest, unittest.TestCase):
    def _get_server(self):
        return TCPRegistryServer(host="127.0.0.1", pruning_timeout=PRUNING_TIMEOUT, allow_listing=True)

    def _get_client(self):
        return TCPRegistryClient(ip="127.0.0.1")


class TestUdpRegistry(BaseRegistryTest, unittest.TestCase):
    """ May fail due to iptables/packet-drops. """
    def _get_server(self):
        return UDPRegistryServer(host="0.0.0.0", pruning_timeout=PRUNING_TIMEOUT, allow_listing=True)

    def _get_client(self):
        return UDPRegistryClient()


if __name__ == "__main__":
    unittest.main()



import rpyc
import os
import time
from threading import Thread


class FileMonitorService(rpyc.Service):
    class exposed_FileMonitor(object):
        def __init__(self, filename, callback, interval=1):
            self.filename = filename
            self.interval = interval
            self.last_stat = None
            self.callback = rpyc.async_(callback)   # make the callback async
            self.active = True
            self.thread = Thread(target=self.work)
            self.thread.start()

        def exposed_stop(self):
            self.active = False
            self.thread.join()

        def work(self):
            while self.active:
                stat = os.stat(self.filename)
                if self.last_stat is not None and self.last_stat != stat:
                    self.callback(self.last_stat, stat)
                self.last_stat = stat
                time.sleep(self.interval)


if __name__ == "__main__":
    from rpyc.utils.server import ThreadedServer
    ThreadedServer(FileMonitorService, port=18871).start()



import rpyc
from rpyc.utils.server import ThreadedServer
from rpyc import SlaveService
import unittest
try:
    import urllib3
    urllib3.request
    _urllib3_import_failed = False
except Exception:
    _urllib3_import_failed = True


@unittest.skipIf(_urllib3_import_failed or True, "urllib3 not available")
class TestUrllib3(unittest.TestCase):
    """ #547 """

    def setUp(self):
        self.cfg = {'sync_request_timeout': 60*60}
        self.server = ThreadedServer(SlaveService, port=18878, auto_register=False, protocol_config=self.cfg)
        self.server.logger.quiet = False
        self.server._start_in_thread()
        self.conn = rpyc.classic.connect('localhost', port=18878)

    def tearDown(self):
        self.conn.close()
        self.server.close()

    def test_issue(self):
        self.conn.execute('import urllib3')
        urllib3_ = self.conn.modules.urllib3
        # headers = urllib3.HTTPHeaderDict()
        # headers.add("Accept", "application/json")
        # headers.add("Accept", "text/plain")
        headers = {"X-Request-Id": "test"}
        resp = urllib3_.request("POST", "https://httpbin.org/post", fields={"hello": "world"}, headers=headers)
        __import__('code').interact(local=locals() | globals())
        # self.assertTrue(self.conn.root.instance(remote_list, list))


if __name__ == "__main__":
    unittest.main()



import rpyc
import gobject
import gtk
import pygtk
pygtk.require('2.0')


class ChatClient(object):
    def __init__(self):
        self.conn = None
        self.user = None

        # -------- GUI CODE ---------
        window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        window.set_resizable(True)
        window.connect("destroy", self.on_close)
        window.set_title("RPyChat")

        top = gtk.VBox()
        top.show()
        window.add(top)
        hbox1 = gtk.HBox()
        hbox1.show()
        top.pack_start(hbox1, expand=False)
        hbox2 = gtk.HBox()
        hbox2.show()
        top.pack_start(hbox2, expand=False)
        self.box_main = gtk.VBox()
        top.pack_start(self.box_main, padding=10)

        lbl_host = gtk.Label("Host:")
        lbl_host.show()
        hbox1.pack_start(lbl_host)
        self.txt_host = gtk.Entry()
        self.txt_host.set_text("127.0.0.1")
        self.txt_host.show()
        self.txt_host.connect("activate", self.on_connect, "via-text")
        hbox1.pack_start(self.txt_host)

        lbl_port = gtk.Label("Port:")
        lbl_port.show()
        hbox1.pack_start(lbl_port, padding=10)
        self.txt_port = gtk.Entry()
        self.txt_port.set_text("19912")
        self.txt_port.show()
        self.txt_port.connect("activate", self.on_connect, "via-text")
        hbox1.pack_start(self.txt_port)

        lbl_user = gtk.Label("Username:")
        lbl_user.show()
        hbox2.pack_start(lbl_user, padding=10)
        self.txt_user = gtk.Entry()
        self.txt_user.show()
        self.txt_user.connect("activate", self.on_connect, "via-text")
        hbox2.pack_start(self.txt_user)

        lbl_password = gtk.Label("Password:")
        lbl_password.show()
        hbox2.pack_start(lbl_password, padding=10)
        self.txt_password = gtk.Entry()
        self.txt_password.set_visibility(False)
        self.txt_password.show()
        self.txt_password.connect("activate", self.on_connect, "via-text")
        hbox2.pack_start(self.txt_password)

        self.btn_connect = gtk.Button("Connect")
        self.btn_connect.show()
        self.btn_connect.connect("clicked", self.on_connect)
        hbox2.pack_start(self.btn_connect, padding=10)

        sw = gtk.ScrolledWindow()
        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
        sw.show()
        self.box_main.pack_start(sw)

        self.txt_main = gtk.TextView()
        self.txt_main.set_editable(False)
        self.txt_main.show()
        sw.add(self.txt_main)

        hbox3 = gtk.HBox()
        hbox3.show()
        self.box_main.pack_start(hbox3, fill=False, expand=False)

        self.txt_input = gtk.Entry()
        self.txt_input.show()
        self.txt_input.connect("activate", self.on_send)
        hbox3.pack_start(self.txt_input)

        self.btn_send = gtk.Button("Send")
        self.btn_send.show()
        self.btn_send.connect("clicked", self.on_send)
        hbox3.pack_start(self.btn_send, fill=False, expand=False, padding=10)

        window.show()
        # -------- END OF GUI CODE ---------

    def disconnect(self):
        if self.conn:
            try:
                self.user.logout()
            except:
                pass
            self.conn.close()
            self.user = None
            self.conn = None

    def on_close(self, widget):
        self.disconnect()
        gtk.main_quit()

    #
    # connect/disconnect logic
    #
    def on_connect(self, widget, data=None):
        if self.btn_connect.get_label() == "Connect":
            try:
                self.conn = rpyc.connect(self.txt_host.get_text(), int(self.txt_port.get_text()))
            except Exception:
                self.conn = None
                m = gtk.MessageDialog(buttons=gtk.BUTTONS_OK,
                                      type=gtk.MESSAGE_ERROR,
                                      flags=gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                                      message_format="Connection refused")
                m.run()
                m.destroy()
                return

            try:
                self.user = self.conn.root.login(self.txt_user.get_text(),
                                                 self.txt_password.get_text(), self.on_message)
            except ValueError:
                self.conn.close()
                self.conn = None
                m = gtk.MessageDialog(buttons=gtk.BUTTONS_OK,
                                      type=gtk.MESSAGE_ERROR,
                                      flags=gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                                      message_format="Invalid Login")
                m.run()
                m.destroy()
                return

            # register in the GTK reactor
            gobject.io_add_watch(self.conn, gobject.IO_IN, self.bg_server)
            self.btn_connect.set_label("Disconnect")
            self.box_main.show()
        else:
            if data == "via-text":
                return
            self.disconnect()
            self.box_main.hide()
            self.btn_connect.set_label("Connect")

    #
    # called by the reactor whenever the connection has something to say
    #
    def bg_server(self, source=None, cond=None):
        if self.conn:
            self.conn.poll_all()
            return True
        else:
            return False

    #
    # sends the current message
    #
    def on_send(self, widget, data=None):
        text = self.txt_input.get_text()
        self.txt_input.set_text("")
        if text.strip():
            self.user.say(text)

    #
    # called by the server, with the text to append to the GUI
    #
    def on_message(self, text):
        buf = self.txt_main.get_buffer()
        buf.place_cursor(buf.get_end_iter())
        buf.insert_at_cursor(text + "\n")
        self.txt_main.scroll_to_iter(buf.get_end_iter(), 0)


if __name__ == "__main__":
    cc = ChatClient()
    gtk.main()



import sys
import rpyc
from pathlib import Path

def load_tests(loader, standard_tests, pattern):
    # Hook rpyc logger, unittest verbosity, and system paths
    #rpyc.core.DEFAULT_CONFIG['logger'] = rpyc.lib.setup_logger()
    rpyc_tests_path = Path(__file__).absolute().parent
    rpyc_path = rpyc_tests_path.parent
    for p in [str(rpyc_path), str(rpyc_tests_path)]:
        if p not in sys.path:
            sys.path.insert(0, p)

    # Discover on tests and add paths
    tests = loader.discover(start_dir=rpyc_tests_path, pattern=pattern, top_level_dir=rpyc_path)
    standard_tests.addTests(tests)
    return standard_tests



import os
import tempfile
import shutil
import unittest
import rpyc


class Test_Remoting(unittest.TestCase):
    def setUp(self):
        self.conn = rpyc.classic.connect_thread()

    def tearDown(self):
        self.conn.close()

    def test_files(self):
        base = tempfile.mkdtemp()
        base1 = os.path.join(base, "1")
        base2 = os.path.join(base, "2")
        base3 = os.path.join(base, "3")
        os.mkdir(base1)
        for i in range(10):
            f = open(os.path.join(base1, f"foofoo{i}"), "w")
            f.close()
        os.mkdir(os.path.join(base1, "somedir1"))

        rpyc.classic.upload(self.conn, base1, base2)
        self.assertEqual(sorted(os.listdir(base1)), sorted(os.listdir(base2)))

        rpyc.classic.download(self.conn, base2, base3)
        self.assertEqual(sorted(os.listdir(base2)), sorted(os.listdir(base3)))

        shutil.rmtree(base)

    @unittest.skip("TODO: upload a package and a module")
    def test_distribution(self):
        pass

    @unittest.skip("Requires manual testing atm")
    def test_interactive(self):
        print("type Ctrl+D to exit (Ctrl+Z on Windows)")
        rpyc.classic.interact(self.conn)

    @unittest.skip("Requires manual testing atm")
    def test_post_mortem(self):
        try:
            self.conn.modules.sys.path[100000]
        except IndexError:
            print("type 'q' to exit")
            rpyc.classic.pm(self.conn)
            raise
        else:
            self.fail("expected an exception")

    def test_migration(self):
        path_list = rpyc.classic.obtain(self.conn.modules.sys.path)
        self.assertTrue(type(path_list) is list)
        rl = rpyc.classic.deliver(self.conn, path_list)
        self.assertTrue(isinstance(rl, rpyc.BaseNetref))


if __name__ == "__main__":
    unittest.main()



import rpyc
from time import sleep


class MyClient(object):

    def __init__(self):
        self.conn = rpyc.connect("localhost", 18000)
        self.bgsrv = rpyc.BgServingThread(self.conn)
        self.root = self.conn.root
        self.service = self.root.MyService("/tmp/test.txt", self.on_event)  # create a filemon

    def on_event(self, oldstat, newstat):
        print("file changed")
        print(f"    old stat: {oldstat}")
        print(f"    new stat: {newstat}")

    def close(self):
        self.service.stop()
        self.bgsrv.stop()
        self.conn.close()


if __name__ == "__main__":

    client = MyClient()
    sleep(10)
    client.close()



from rpyc.lib.compat import BYTES_LITERAL
from rpyc.core.stream import PipeStream, NamedPipeStream
import rpyc
import sys
import time
import unittest


@unittest.skipIf(sys.platform != "win32", "Requires windows")
class TestPipes(unittest.TestCase):
    def test_basic_io(self):
        p1, p2 = PipeStream.create_pair()
        p1.write(BYTES_LITERAL("hello"))
        assert p2.poll(0)
        assert p2.read(5) == BYTES_LITERAL("hello")
        assert not p2.poll(0)
        p2.write(BYTES_LITERAL("world"))
        assert p1.poll(0)
        assert p1.read(5) == BYTES_LITERAL("world")
        assert not p1.poll(0)
        p1.close()
        p2.close()

    def test_rpyc(self):
        p1, p2 = PipeStream.create_pair()
        client = rpyc.connect_stream(p1)
        server = rpyc.connect_stream(p2)
        server_thread = rpyc.spawn(server.serve_all)
        assert client.root.get_service_name() == "VOID"
        t = rpyc.BgServingThread(client)
        assert server.root.get_service_name() == "VOID"
        t.stop()
        client.close()
        server.close()
        server_thread.join()


@unittest.skipIf(sys.platform != "win32", "Requires windows")
class TestNamedPipe(unittest.TestCase):
    def setUp(self):
        self.pipe_server_thread = rpyc.spawn(self.pipe_server)
        time.sleep(1)  # make sure server is accepting already
        self.np_client = NamedPipeStream.create_client("floop")
        self.client = rpyc.connect_stream(self.np_client)

    def tearDown(self):
        self.client.close()
        self.server.close()
        self.pipe_server_thread.join()

    def pipe_server(self):
        self.np_server = NamedPipeStream.create_server("floop")
        self.server = rpyc.connect_stream(self.np_server)
        self.server.serve_all()

    def test_rpyc(self):
        assert self.client.root.get_service_name() == "VOID"
        t = rpyc.BgServingThread(self.client)
        assert self.server.root.get_service_name() == "VOID"
        t.stop()


if __name__ == "__main__":
    unittest.main()



from __future__ import with_statement
import rpyc
import unittest

from contextlib import contextmanager


class MyService(rpyc.Service):

    on_context_enter = False
    on_context_exit = False
    on_context_exc = False

    @contextmanager
    def exposed_context(self, y):
        self.on_context_enter = True
        try:
            yield 17 + y
        except Exception:
            self.on_context_exc = True
            raise
        finally:
            self.on_context_exit = True


class TestContextManagers(unittest.TestCase):
    def setUp(self):
        self.service = MyService()
        self.conn = rpyc.connect_thread(remote_service=self.service)

    def tearDown(self):
        self.conn.close()

    def test_context(self):
        service = self.service
        with self.conn.root.context(3) as x:
            self.assertTrue(service.on_context_enter)
            self.assertFalse(service.on_context_exc)
            self.assertFalse(service.on_context_exit)
            self.assertEqual(x, 20)
        self.assertFalse(service.on_context_exc)
        self.assertTrue(service.on_context_exit)

    def test_context_exception(self):
        class MyException(Exception):
            pass

        service = self.service

        def use_context():
            with self.conn.root.context(3):
                self.assertTrue(service.on_context_enter)
                self.assertFalse(service.on_context_exc)
                self.assertFalse(service.on_context_exit)
                raise MyException()

        self.assertRaises(MyException, use_context)

        self.assertTrue(service.on_context_exc)
        self.assertTrue(service.on_context_exit)


if __name__ == "__main__":
    unittest.main()



import rpyc
import os
from threading import Thread
import time


class MyServiceFactory(rpyc.Service):

    class exposed_MyService(object):   # exposing names is not limited to methods :)

        def __init__(self, filename, callback, interval=1):
            print("news client with " + filename + " " + str(callback))
            self.filename = filename
            self.interval = interval
            self.last_stat = None
            self.callback = rpyc.async_(callback)   # create an async callback
            self.active = True
            self.thread = Thread(target=self.work)
            self.thread.start()

        def exposed_stop(self):   # this method has to be exposed too
            self.active = False
            self.thread.join()

        def work(self):
            while self.active:
                stat = os.stat(self.filename)
                if self.last_stat is not None and self.last_stat != stat:
                    self.callback(self.last_stat, stat)   # notify the client of the change
                self.last_stat = stat
                time.sleep(self.interval)
            print("client closed.")



import rpyc
from rpyc.utils.server import OneShotServer
import unittest


class MyService(rpyc.Service):

    def exposed_foo(self):
        return "bar"


class Test_OneShotServer(unittest.TestCase):

    def setUp(self):
        self.server = OneShotServer(MyService, port=18878, auto_register=False)
        self.server.logger.quiet = False
        self.thd = self.server._start_in_thread()

    def tearDown(self):
        self.server.close()
        self.thd.join()

    def test_server_stops(self):
        conn = rpyc.connect("localhost", port=18878)
        self.assertEqual("bar", conn.root.foo())
        conn.close()
        while not self.server._closed:
            pass
        self.assertTrue(self.server._closed)
        self.assertTrue(self.server.listener._closed)
        self.assertEqual(len(self.server.clients), 0)


if __name__ == "__main__":
    unittest.main()



import pathlib
import rpyc
import subprocess
import sys
import tempfile
import unittest
import os
from rpyc.utils.server import ThreadedServer
from shutil import which


class ParentGDB(rpyc.Service):
    """ starts a new gdb service instance on connect and quits on disconnect """

    def on_connect(self, conn):
        tests_path = pathlib.Path(__file__).resolve().parent
        gdb_cmd = ['gdb', '-q', '-x', pathlib.Path(tests_path, 'gdb_service.py')]
        env = os.environ.copy()
        env['PYTHONPATH'] = ':'.join(sys.path)
        self._proc = subprocess.Popen(gdb_cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, env=env)
        stdout = self._proc.stdout.readline()
        self._gdb_svc_port = int(stdout.strip().decode())
        print(self._gdb_svc_port)
        self.gdb_svc_conn = rpyc.connect(host='localhost', port=self._gdb_svc_port)

    def on_disconnect(self, conn):
        self._proc.communicate()
        self._proc.kill()
        self.gdb_svc_conn.root.quit()
        self.gdb_svc_conn.close()

    def exposed_get_gdb(self):
        return self.gdb_svc_conn.root.get()


@unittest.skipUnless(which('gdb') is not None, "Skipping gdb example test since gdb not found")
class Test_GDB(unittest.TestCase):

    def setUp(self):
        self.dtemp = tempfile.mkdtemp()
        self.a_out = pathlib.Path(self.dtemp, 'a.out')
        compile_cmd = ['g++', '-g', '-o', str(self.a_out), '-x', 'c++', '-']
        proc = subprocess.Popen(compile_cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, stdin=subprocess.PIPE)
        proc_input = b'int func(int a, int b){return a + b;}int main(){return func(1, 2);}'
        stdout, stderr = proc.communicate(input=proc_input)
        if stdout or stderr:
            raise ValueError("stdout and stderr should have be empty for a.out creation")
        self.server = ThreadedServer(ParentGDB, port=18878, auto_register=False,
                                     protocol_config={'allow_all_attrs': True})
        self.server._start_in_thread()

    def tearDown(self):
        self.server.close()
        while not self.server._closed:
            pass

    def test_gdb(self):
        print(0)
        parent_gdb_conn = rpyc.connect(host='localhost', port=18878)
        print(1)
        gdb = parent_gdb_conn.root.get_gdb()
        print(2)
        gdb.execute('file {}'.format(self.a_out))
        print(3)
        disasm = gdb.execute('disassemble main', to_string=True)
        print(4)
        self.assertIn('End of assembler dump', disasm)
        parent_gdb_conn.close()


if __name__ == "__main__":
    unittest.main()



from __future__ import with_statement
from rpyc.utils.classic import teleport_function
from rpyc.lib.compat import is_py_3k, is_py_gte38, is_py_gte311
from rpyc.utils.teleportation import export_function, import_function
import subprocess
import sys
import os
import rpyc
import types
import unittest
import tracemalloc
tracemalloc.start()


def b(st):
    if sys.version_info[0] >= 3:
        return bytes(st, "latin-1")
    else:
        return st


def f(a):
    def g(b):
        return a + int(b)
    return g


def defaults(a=5, b="hi", c=(5.5, )):
    return a, b, c


def kwdefaults(pos=5, *, a=42, b="bye", c=(12.4, )):
    return pos, a, b, c


def h(a):
    import os
    return a * os.getpid()


def foo():
    return bar() + 1


def bar():
    return 42


class TeleportationTest(unittest.TestCase):
    def setUp(self):
        server_file = os.path.join(os.path.dirname(os.path.abspath(__file__)), "..", "bin", "rpyc_classic.py")
        self.proc = subprocess.Popen([sys.executable, server_file, "--mode=oneshot", "--host=localhost", "-p0"],
                                     stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        line = self.proc.stdout.readline().strip()
        if not line:
            print(self.proc.stderr.read())
            self.fail("server failed to start")
        self.assertEqual(line, b("rpyc-oneshot"), "server failed to start")
        host, port = self.proc.stdout.readline().strip().split(b("\t"))
        self.conn = rpyc.classic.connect(host, int(port))

    def tearDown(self):
        self.conn.close()
        self.proc.communicate()  # clear io so resources are closed

    def test(self):
        exp = export_function(f)
        f2 = import_function(exp)
        self.assertEqual(f(6)(7), f2(6)(7))

        h2 = teleport_function(self.conn, h)
        self.assertNotEqual(h(7), h2(7))

    def test_globals(self):
        def the_answer():
            return THE_ANSWER  # noqa

        teleported = teleport_function(self.conn, the_answer)
        self.conn.namespace['THE_ANSWER'] = 42
        self.assertEqual(teleported(), 42)

        the_globals = self.conn.builtins.dict({'THE_ANSWER': 43})
        teleported2 = teleport_function(self.conn, the_answer, the_globals)
        self.assertEqual(teleported2(), 43)

    def test_def(self):
        foo_ = teleport_function(self.conn, foo)
        bar_ = teleport_function(self.conn, bar)
        self.assertEqual(foo_(), 43)
        self.assertEqual(bar_(), 42)

    def test_defaults(self):
        defaults_ = teleport_function(self.conn, defaults)
        self.assertEqual(defaults_(), defaults())

    def test_kwdefaults(self):
        kwdefaults_ = teleport_function(self.conn, kwdefaults)
        self.assertEqual(kwdefaults_(), kwdefaults())

    def test_compat(self):  # assumes func has only brineable types

        def get37_schema(cobj):
            return (cobj.co_argcount, 0, cobj.co_nlocals, cobj.co_stacksize,
                    cobj.co_flags, cobj.co_code, cobj.co_consts, cobj.co_names, cobj.co_varnames,
                    cobj.co_filename, cobj.co_name, cobj.co_firstlineno, cobj.co_lnotab,
                    cobj.co_freevars, cobj.co_cellvars)

        def get38_schema(cobj):
            return (cobj.co_argcount, 2, cobj.co_kwonlyargcount, cobj.co_nlocals,
                    cobj.co_stacksize, cobj.co_flags, cobj.co_code, cobj.co_consts, cobj.co_names,
                    cobj.co_varnames, cobj.co_filename, cobj.co_name, cobj.co_firstlineno, cobj.co_lnotab,
                    cobj.co_freevars, cobj.co_cellvars)

        def get311_schema(cobj):
            return (cobj.co_argcount, 2, cobj.co_kwonlyargcount, cobj.co_nlocals,
                    cobj.co_stacksize, cobj.co_flags, cobj.co_code, cobj.co_consts, cobj.co_names,
                    cobj.co_varnames, cobj.co_filename, cobj.co_name, cobj.co_qualname,
                    cobj.co_firstlineno, cobj.co_lnotab, cobj.co_exceptiontable,
                    cobj.co_freevars, cobj.co_cellvars)

        if is_py_gte38:
            def pow37(x, y): return x ** y  # noqa
            def pow38(x, y): return x ** y  # noqa
            export37 = get37_schema(pow37.__code__)
            export38 = get38_schema(pow38.__code__)
            schema37 = (pow37.__name__, pow37.__module__, pow37.__defaults__, pow37.__kwdefaults__, export37)
            schema38 = (pow38.__name__, pow38.__module__, pow38.__defaults__, pow38.__kwdefaults__, export38)
            pow37_netref = self.conn.modules["rpyc.utils.teleportation"].import_function(schema37)
            pow38_netref = self.conn.modules["rpyc.utils.teleportation"].import_function(schema38)
            self.assertEqual(pow37_netref(2, 3), pow37(2, 3))
            self.assertEqual(pow38_netref(2, 3), pow38(2, 3))
            self.assertEqual(pow37_netref(x=2, y=3), pow37(x=2, y=3))
            if is_py_gte38 and not is_py_gte311:
                pow38.__code__ = types.CodeType(*export38)  # pow38 = lambda x, y, /: x ** y
                with self.assertRaises(TypeError):  # show local behavior
                    pow38(x=2, y=3)
                with self.assertRaises(TypeError):
                    pow38_netref(x=2, y=3)
            if is_py_gte311:
                def pow311(x, y): return x ** y  # noqa
                export311 = get311_schema(pow311.__code__)
                schema311 = (pow311.__name__, pow311.__module__, pow311.__defaults__, pow311.__kwdefaults__, export311)
                pow311_netref = self.conn.modules["rpyc.utils.teleportation"].import_function(schema311)
                self.assertTrue(is_py_gte311)
                pow311.__code__ = types.CodeType(*export311)  # pow311 = lambda x, y, /: x ** y
                with self.assertRaises(TypeError):  # show local behavior
                    pow311(x=2, y=3)
                with self.assertRaises(TypeError):
                    pow311_netref(x=2, y=3)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
"""Shows expected behavior for a client when the remote thread serving this client is busy/sleeping.

Additional context: https://github.com/tomerfiliba-org/rpyc/issues/491#issuecomment-1131843406
"""
import logging
import threading
import time
import rpyc


logger = rpyc.setup_logger(namespace='client')
rpyc.core.protocol.DEFAULT_CONFIG['logger'] = logger


def async_example(connection, event):
    _async_function = rpyc.async_(connection.root.function)  # create async proxy
    # The server will call event.wait which will block this thread. To process
    # the set message from the server we need a background thread. A background
    # thread ensures that we have a thread that is not blocked.
    #
    # But wait! Since the communication is symmetric, the server side could
    # be blocked if you are not careful. It needs responses from the client
    #
    # The perils of trying to thread a single connection...
    # - the thread the receives the message from the server to wait is blocked
    # - which thread is blocked is VERY hard to guarantee
    #
    # THIS IS NOT HE PREFERRED WAY FOR MUTABLE TYPES...
    # - threading a connection might be okay to do for immutable types depending on context

    bgsrv = rpyc.BgServingThread(connection)
    ares = _async_function(event, block_server_thread=False)
    value = ares.value
    event.clear()
    logger.info('Running buggy blocking example...')
    ares = _async_function(event, block_server_thread=True)
    value = ares.value
    event.clear()
    bgsrv.stop()


def how_to_block_main_thread(connection, event):
    """Example of how to block the main thread of a client"""
    t0 = time.time()
    logger.debug("Running example that blocks main thread of client...")
    value = connection.root.function(event, call_set=True)
    logger.debug(f"Value returned after {time.time()-t0}s: {value}")


class Event:
    def __init__(self):
        self._evnt = threading.Event()

    def __getattr__(self, name):
        if name in ('wait', 'set', 'clear'):
            logging.info(f'Event.__getattr__({name})')
        return getattr(self._evnt, name)


if __name__ == "__main__":
    logger.info('Printed from main thread')
    connection = rpyc.connect("localhost", 18812, config=dict(allow_all_attrs=True))
    event = Event()
    async_example(connection, event)
    event.clear()
    # how_to_block_main_thread_example(connection, event)



"""Supporting functions for unit tests

The core logic of the functions `_ignore_deprecated_imports` and `import_module` is from the cpython code base:
- https://github.com/python/cpython/blob/da576e08296490e94924421af71001bcfbccb317/Lib/test/support/import_helper.py
"""
import warnings
import sys
import contextlib
import unittest


@contextlib.contextmanager
def _ignore_deprecated_imports(ignore=True):
    """Context manager to suppress package and module deprecation
    warnings when importing them.
    If ignore is False, this context manager has no effect.
    """
    if ignore:
        with warnings.catch_warnings():
            warnings.filterwarnings("ignore", ".+ (module|package)",
                                    DeprecationWarning)
            yield
    else:
        yield


def import_module(name, deprecated=False, *, required_on=(), fromlist=()):
    """Import and return the module to be tested, raising SkipTest if
    it is not available.
    If deprecated is True, any module or package deprecation messages
    will be suppressed. If a module is required on a platform but optional for
    others, set required_on to an iterable of platform prefixes which will be
    compared against sys.platform.
    """
    with _ignore_deprecated_imports(deprecated):
        try:
            module = __import__(name, fromlist=fromlist)
            for a in fromlist:
                if not hasattr(module, a):
                    raise ImportError(f"cannot import name '{a}' from '{name}'")
            return module
        except ImportError as msg:
            if sys.platform.startswith(tuple(required_on)):
                raise
            raise unittest.SkipTest(str(msg))



import os
import rpyc
import tempfile
from rpyc.utils.server import ThreadedServer, ThreadPoolServer
from rpyc import SlaveService
import unittest


class Test_ThreadedServer(unittest.TestCase):

    def setUp(self):
        self.server = ThreadedServer(SlaveService, port=18878, auto_register=False)
        self.server.logger.quiet = False
        self.server._start_in_thread()

    def tearDown(self):
        while self.server.clients:
            pass
        self.server.close()

    def test_connection(self):
        conn = rpyc.classic.connect("localhost", port=18878)
        print(conn.modules.sys)
        print(conn.modules["xml.dom.minidom"].parseString("<a/>"))
        conn.execute("x = 5")
        self.assertEqual(conn.namespace["x"], 5)
        self.assertEqual(conn.eval("1+x"), 6)
        conn.close()


class Test_ThreadedServerOverUnixSocket(unittest.TestCase):

    def setUp(self):
        self.socket_path = tempfile.mktemp()
        self.server = ThreadedServer(SlaveService, socket_path=self.socket_path, auto_register=False)
        self.server.logger.quiet = False
        self.server._start_in_thread()

    def tearDown(self):
        self.server.close()
        os.remove(self.socket_path)

    def test_connection(self):
        c = rpyc.classic.unix_connect(self.socket_path)
        print(c.modules.sys)
        print(c.modules["xml.dom.minidom"].parseString("<a/>"))
        c.execute("x = 5")
        self.assertEqual(c.namespace["x"], 5)
        self.assertEqual(c.eval("1+x"), 6)
        c.close()


class Test_ThreadPoolServer(Test_ThreadedServer):

    def setUp(self):
        self.server = ThreadPoolServer(SlaveService, port=18878, auto_register=False)
        self.server.logger.quiet = False
        self.server._start_in_thread()


if __name__ == "__main__":
    unittest.main()



import rpyc
import unittest


class Properties(object):
    def __init__(self):
        self._x = 0

    @property
    def counter(self):
        self._x += 1
        return self._x

    @property
    def dont_touch_me(self):
        # reconstruct bug reported by Andrew Stromnov
        # http://groups.google.com/group/rpyc/msg/aa6110259481f194
        1 / 0


class TestAttributes(unittest.TestCase):
    def setUp(self):
        self.conn = rpyc.classic.connect_thread()

    def tearDown(self):
        self.conn.close()

    def test_properties(self):
        p = self.conn.modules["tests.test_attributes"].Properties()
        print(p.counter)  # 1
        print(p.counter)  # 2
        print(p.counter)  # 3
        self.assertEqual(p.counter, 4)  # 4


if __name__ == "__main__":
    unittest.main()



import rpyc
import rpyc.core.async_ as rc_async_
import rpyc.core.protocol as rc_protocol
import contextlib
import signal
import threading
import time
import unittest


class TestRace(unittest.TestCase):
    def setUp(self):
        self.connection = rpyc.classic.connect_thread()

        self.a_str = rpyc.async_(self.connection.builtin.str)

    def tearDown(self):
        self.connection.close()

    def test_asyncresult_race(self):
        with _patch():
            def hook():
                time.sleep(0.2)  # loose race

            _AsyncResult._HOOK = hook

            threading.Thread(target=self.connection.serve_all).start()
            time.sleep(0.1)  # wait for thread to serve

            # schedule KeyboardInterrupt
            thread_id = threading.get_ident()
            _ = lambda: signal.pthread_kill(thread_id, signal.SIGINT)
            timer = threading.Timer(1, _)
            timer.start()

            a_result = self.a_str("")  # request
            time.sleep(0.1)  # wait for race to start
            try:
                a_result.wait()
            except KeyboardInterrupt:
                raise Exception("deadlock")

            timer.cancel()


class _AsyncResult(rc_async_.AsyncResult):
    _HOOK = None

    def __call__(self, *args, **kwargs):
        hook = type(self)._HOOK
        if hook is not None:
            hook()
        return super().__call__(*args, **kwargs)


@contextlib.contextmanager
def _patch():
    AsyncResult = rc_async_.AsyncResult
    try:
        rc_async_.AsyncResult = _AsyncResult
        rc_protocol.AsyncResult = _AsyncResult  # from import
        yield

    finally:
        rc_async_.AsyncResult = AsyncResult
        rc_protocol.AsyncResult = AsyncResult


if __name__ == "__main__":
    unittest.main()



import unittest
from rpyc import SlaveService
from rpyc.utils.server import GeventServer
import time
import rpyc
try:
    import gevent
    _gevent_import_failed = False
except Exception:
    _gevent_import_failed = True


@unittest.skipIf(_gevent_import_failed, "Gevent is not available")
class Test_GeventServer(unittest.TestCase):

    def setUp(self):
        from gevent import monkey
        monkey.patch_all()
        self.server = GeventServer(SlaveService, port=18878, auto_register=False)
        self.server.logger.quiet = False
        self.server._listen()
        gevent.spawn(self.server.start)

    def tearDown(self):
        self.server.close()

    def test_connection(self):
        with rpyc.classic.connect("localhost", port=18878) as c:
            c.execute("x = 5")
            self.assertEqual(c.namespace["x"], 5)
            self.assertEqual(c.eval("1+x"), 6)

    def test_multiple_connections(self):
        def get_ident(gevent):
            return gevent.monkey.get_original('threading', 'get_ident')()
        c1 = rpyc.classic.connect("localhost", port=18878)
        c2 = rpyc.classic.connect("localhost", port=18878)
        c3 = rpyc.classic.connect("localhost", port=18878)
        with c1, c2, c3:
            id0 = get_ident(gevent)
            id1 = get_ident(c1.modules.gevent)
            id2 = get_ident(c2.modules.gevent)
            id3 = get_ident(c3.modules.gevent)
            # all server greenlets and clients running in same OS thread ;)
            self.assertEqual(id0, id1)
            self.assertEqual(id1, id2)
            self.assertEqual(id1, id3)

    def test_parallelism(self):
        conns = [rpyc.classic.connect("localhost", port=18878)
                 for _ in range(50)]
        try:
            start = time.time()
            gevent.joinall([
                gevent.spawn(c.modules.time.sleep, 1)
                for c in conns
            ])
            stop = time.time()

            self.assertLessEqual(stop - start, 2)

        finally:
            for c in conns:
                c.close()


if __name__ == "__main__":
    unittest.main()



import rpyc
from rpyc.utils.server import ThreadedServer
import unittest

CONNECT_CONFIG = {"allow_setattr": True}


class Fee(object):

    def __init__(self, msg="Fee"):
        self._msg = msg

    @property
    def exposed_msg(self):
        return self._msg

    @exposed_msg.setter
    def exposed_msg(self, value):
        self._msg = value

    def __str__(self):
        return str(self._msg)

    def __add__(self, rhs):
        return self.__str__() + str(rhs)


class Service(rpyc.Service):

    PORT = 18878

    def exposed_fee(self, arg):
        return arg

    def exposed_fee_str(self, arg):
        return str(arg)

    def exposed_foe_update(self, arg, msg):
        arg.msg = arg.msg + " foe" + msg
        return arg


class Intermediate(rpyc.Service):

    PORT = 18879

    def exposed_fee(self, arg):
        with rpyc.connect("localhost", port=Service.PORT, config=CONNECT_CONFIG) as conn:
            return conn.root.fee(arg)

    def exposed_fee_str(self, arg):
        with rpyc.connect("localhost", port=Service.PORT) as conn:
            return conn.root.fee_str(arg)

    def exposed_fie_update(self, arg):
        arg.msg = arg.msg + " fie"
        with rpyc.connect("localhost", port=Service.PORT, config=CONNECT_CONFIG) as conn:
            return conn.root.foe_update(arg, " foo bar")


class Test_rpyc_over_rpyc(unittest.TestCase):
    """Issue #346 shows that exceptions are being raised when an RPyC service method
    calls another RPyC service, forwarding a non-trivial (and thus given as a proxy) argument.
    """

    def setUp(self):
        self.server = ThreadedServer(Service, port=Service.PORT, auto_register=False)
        self.i_server = ThreadedServer(Intermediate, port=Intermediate.PORT,
                                       auto_register=False, protocol_config=CONNECT_CONFIG)
        self.server._start_in_thread()
        self.i_server._start_in_thread()
        self.conn = rpyc.connect("localhost", port=Intermediate.PORT, config=CONNECT_CONFIG)

    def tearDown(self):
        self.conn.close()
        while self.server.clients or self.i_server.clients:
            pass
        self.server.close()
        self.i_server.close()

    def test_immutable_object_return(self):
        """Tests using rpyc over rpyc---issue #346 reported traceback for this use case"""
        obj = Fee()
        result = self.conn.root.fee_str(obj)
        self.assertEqual(str(obj), "Fee", "String representation of obj should not have changed")
        self.assertEqual(str(result), "Fee", "String representation of result should be the same as obj")

    def test_return_of_unmodified_parameter(self):
        obj = Fee()
        original_obj_id = id(obj)
        result = self.conn.root.fee(obj)
        self.assertEqual(str(obj), "Fee", "String representation of obj should not have changed")
        self.assertEqual(id(result), original_obj_id, "Unboxing of result should be bound to the same object as obj")

    def test_return_of_modified_parameter(self):
        obj = Fee()
        original_obj_id = id(obj)
        result = self.conn.root.fie_update(obj)
        self.assertEqual(str(obj), "Fee fie foe foo bar", "String representation of obj should have changed")
        self.assertEqual(id(result), original_obj_id, "Unboxing of result should be bound to the same object as obj")


if __name__ == "__main__":
    unittest.main()



import sys
import rpyc
import unittest

is_py3 = sys.version_info >= (3,)


class Meta(type):

    def __hash__(self):
        return 4321


Base = Meta('Base', (object,), {})


class Foo(Base):
    def __hash__(self):
        return 1234


class Bar(Foo):
    pass


class Mux(Foo):
    def __eq__(self, other):
        return True


class TestContextManagers(unittest.TestCase):
    def setUp(self):
        self.conn = rpyc.classic.connect_thread()

    def tearDown(self):
        self.conn.close()

    def test_hash_class(self):
        hesh = self.conn.builtins.hash
        mod = self.conn.modules.tests.test_magic
        self.assertEqual(hash(mod.Base), 4321)
        self.assertEqual(hash(mod.Foo), 4321)
        self.assertEqual(hash(mod.Bar), 4321)
        self.assertEqual(hash(mod.Base().__class__), 4321)
        self.assertEqual(hash(mod.Foo().__class__), 4321)
        self.assertEqual(hash(mod.Bar().__class__), 4321)

        basecl_ = mod.Foo().__class__.__mro__[1]
        object_ = mod.Foo().__class__.__mro__[2]
        self.assertEqual(hash(basecl_), hesh(basecl_))
        self.assertEqual(hash(object_), hesh(object_))
        self.assertEqual(hash(object_), hesh(self.conn.builtins.object))

    def test_hash_obj(self):
        hesh = self.conn.builtins.hash
        mod = self.conn.modules.tests.test_magic
        obj = mod.Base()

        self.assertNotEqual(hash(obj), 1234)
        self.assertNotEqual(hash(obj), 4321)
        self.assertEqual(hash(obj), hesh(obj))

        self.assertEqual(hash(mod.Foo()), 1234)
        self.assertEqual(hash(mod.Bar()), 1234)
        if is_py3:
            # py3 implicitly adds '__hash__=None' during class construction
            # if '__eq__ is defined:
            self.assertRaises(TypeError, lambda: hash(mod.Mux()))
        else:
            self.assertEqual(hash(mod.Mux()), 1234)


if __name__ == "__main__":
    unittest.main()



import math
import time

import rpyc
import unittest


class MyMeta(type):
    def spam(self):
        return self.__name__ * 5


class MyClass(object):
    __metaclass__ = MyMeta


if not isinstance(MyMeta, MyMeta):
    # python 3 compatibility
    MyClass = MyMeta(MyClass.__name__, MyClass.__bases__, dict(MyClass.__dict__))


@rpyc.service
class MyService(rpyc.Service):
    on_connect_called = False
    on_disconnect_called = False
    on_about_to_close_called = False

    def on_connect(self, conn):
        self.on_connect_called = True

    def on_disconnect(self, conn):
        self.on_disconnect_called = True

    def exposed_on_about_to_close(self):
        self.on_about_to_close_called = True

    def exposed_distance(self, p1, p2):
        x1, y1 = p1
        x2, y2 = p2
        return math.sqrt((x2 - x1) ** 2 + (y2 - y1) ** 2)

    def exposed_getlist(self):
        return [1, 2, 3]

    def foobar(self):
        assert False

    def exposed_getmeta(self):
        return MyClass()

    def exposed_instance(self, inst, cls):
        return isinstance(inst, cls)

    @rpyc.exposed
    class MyClass(object):
        def __init__(self, a, b):
            self.a = a
            self.b = b

        @rpyc.exposed
        def foo(self):
            return self.a + self.b

    @rpyc.exposed
    def get_decorated(self):
        return "decorated"

    @rpyc.exposed('prefix_')
    def get_decorated_prefix(self):
        return "decorated_prefix"


def before_closed(root):
    root.on_about_to_close()


class TestCustomService(unittest.TestCase):
    config = {}

    def setUp(self):
        self.service = MyService()
        client_config = {"before_closed": before_closed, "close_catchall": False}
        prefixed_client_config = {'exposed_prefix': 'prefix_'}
        self.conn = rpyc.connect_thread(remote_service=self.service, config=client_config)
        self.prefixed_conn = rpyc.connect_thread(remote_service=self.service,
                                                 config=prefixed_client_config,
                                                 remote_config=prefixed_client_config)
        self.conn.root  # this will block until the service is initialized,
        self.prefixed_conn.root  # this will block until the service is initialized,
        # so we can be sure on_connect_called is True by that time
        self.assertTrue(self.service.on_connect_called)

    def tearDown(self):
        if not self.conn.closed:
            self.conn.close()
        if not self.prefixed_conn.closed:
            self.prefixed_conn.close()
        time.sleep(0.5)  # this will wait a little, making sure
        # on_disconnect_called is already True
        self.assertTrue(self.service.on_disconnect_called)

    def test_before_closed(self):
        self.assertFalse(self.service.on_about_to_close_called)
        self.conn.close()
        self.assertTrue(self.service.on_about_to_close_called)

    def test_aliases(self):
        print(f"service name: {self.conn.root.get_service_name()}")

    def test_distance(self):
        assert self.conn.root.distance((2, 7), (5, 11)) == 5

    def test_attributes(self):
        self.conn.root.distance
        self.conn.root.exposed_distance
        self.conn.root.getlist
        self.conn.root.exposed_getlist
        # this is not an exposed attribute:
        self.assertRaises(AttributeError, lambda: self.conn.root.foobar())
        # methods exposed using decorator
        self.conn.root.get_decorated
        self.conn.root.exposed_get_decorated
        self.prefixed_conn.root.get_decorated_prefix
        self.prefixed_conn.root.prefix_get_decorated_prefix
        self.assertFalse(hasattr(self.conn.root, 'get_decorated_prefix'))
        smc = self.conn.root.MyClass('a', 'b')
        self.assertEqual(smc.foo(), 'ab')

    def test_safeattrs(self):
        x = self.conn.root.getlist()
        # self.require(x == [1, 2, 3]) -- can't compare remote objects, sorry
        # self.require(x * 2 == [1, 2, 3, 1, 2, 3])
        self.assertEqual([y * 2 for y in x], [2, 4, 6])

    def test_metaclasses(self):
        x = self.conn.root.getmeta()
        print(x)

    def test_instancecheck_list(self):
        remote_list = self.conn.root.getlist()
        self.assertTrue(self.conn.root.instance(remote_list, list))


if __name__ == "__main__":
    unittest.main()



import rpyc
import gc
import unittest


class TestRefcount(unittest.TestCase):
    def setUp(self):
        self.conn = rpyc.classic.connect_thread()

    def tearDown(self):
        self.conn.close()

    def test_refcount(self):
        self.conn.execute("""
deleted_objects = []

class DummyObject(object):
    def __init__(self, name):
        self.name = name
    def __del__(self):
        deleted_objects.append(self.name)""")
        rDummyObject = self.conn.namespace["DummyObject"]
        d1 = rDummyObject("d1")
        d2 = rDummyObject("d2")
        d3 = rDummyObject("d3")
        d4 = rDummyObject("d4")  # noqa
        d2_copy = d2
        del d1
        del d3
        gc.collect()
        self.assertEqual(set(self.conn.namespace["deleted_objects"]), set(["d1", "d3"]))
        del d2
        gc.collect()
        self.assertEqual(set(self.conn.namespace["deleted_objects"]), set(["d1", "d3"]))
        del d2_copy
        gc.collect()
        self.assertEqual(set(self.conn.namespace["deleted_objects"]), set(["d1", "d2", "d3"]))


if __name__ == "__main__":
    unittest.main()



import os
import rpyc
import unittest


class ClassicMode(unittest.TestCase):
    def setUp(self):
        self.conn = rpyc.classic.connect_thread()

    def tearDown(self):
        self.conn.close()
        self.conn = None

    def test_piped_server(self):
        # this causes the following lines to be printed to stderr on Windows:
        #
        # close failed in file object destructor:
        #     IOError: [Errno 9] Bad file descriptor
        # close failed in file object destructor:
        #     IOError: [Errno 9] Bad file descriptor
        #
        # this is because the pipe objects that hold the child process' stdin
        # and stdout were disowned by Win32PipeStream (it forcefully takes
        # ownership of the file handles). so when the low-level pipe objects
        # are gc'ed, they cry that their fd is already closed. this is all
        # considered harmless, but there's no way to disable that message
        # to stderr
        server_file = os.path.join(os.path.dirname(os.path.abspath(__file__)), "..", "bin", "rpyc_classic.py")
        conn = rpyc.classic.connect_subproc(server_file)
        conn.modules.sys.path.append("xxx")
        self.assertEqual(conn.modules.sys.path.pop(-1), "xxx")
        conn.close()
        self.assertEqual(conn.proc.wait(), 0)

    def test_buffiter(self):
        bi = rpyc.buffiter(self.conn.builtin.range(10000))
        self.assertEqual(list(bi), list(range(10000)))

    def test_classic(self):
        self.conn.execute("x = 5")
        self.assertEqual(self.conn.namespace["x"], 5)
        self.assertEqual(self.conn.eval("1+x"), 6)

    def test_mock_connection(self):
        from rpyc.utils.classic import MockClassicConnection
        import sys
        import xml.dom.minidom
        conn = MockClassicConnection()
        self.assertTrue(conn.modules.sys is sys)
        self.assertTrue(conn.modules["xml.dom.minidom"].Element is xml.dom.minidom.Element)
        self.assertTrue(conn.builtin.open is open)
        self.assertEqual(conn.eval("2+3"), 5)

    def test_modules(self):
        self.assertIn('tests.test_magic', self.conn.modules)
        self.assertNotIn('test_badmagic', self.conn.modules)
        self.assertIsNone(self.conn.builtins.locals()['self']._last_traceback)


if __name__ == "__main__":
    unittest.main()



import rpyc
import sys
import os
import unittest
from rpyc.utils.server import ThreadedServer
from rpyc import SlaveService, MasterService


try:
    from plumbum.machines.ssh_machine import SshMachine
    localhost_machine = SshMachine("localhost")
    localhost_machine.close()
except Exception:
    localhost_machine = None


@unittest.skipIf(localhost_machine is None, "Requires paramiko_machine to localhost")
class Test_Ssh(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        if sys.platform == "win32":
            cls.server = None
            os.environ["HOME"] = os.path.expanduser("~")
        else:
            # assume "ssh localhost" is configured to run without asking for password
            # `.ssh/config`
            # Host localhost
            #   HostName 127.0.0.1
            #   User <username>
            #   IdentityFile <id_rsa>
            cls.server = ThreadedServer(SlaveService, hostname="localhost",
                                        ipv6=False, port=18888, auto_register=False)
            cls.server._start_in_thread()
        cls.remote_machine = SshMachine("localhost")
        cls.conn = rpyc.classic.ssh_connect(cls.remote_machine, 18888)
        cls.conn2 = rpyc.ssh_connect(cls.remote_machine, 18888, service=MasterService)

    @classmethod
    def tearDownClass(cls):
        cls.conn.close()
        cls.conn2.close()
        # TODO review "ResourceWarning: subprocess 111785 is still running"
        cls.remote_machine._session.proc.terminate()  # fix resource warning
        cls.remote_machine._session.proc.communicate()  # fix resource warning
        cls.remote_machine.close()
        cls.server.close()

    def test_simple(self):
        print("server's pid =", self.conn.modules.os.getpid())
        self.conn.modules.sys.stdout.write("hello over ssh\n")
        self.conn.modules.sys.stdout.flush()

    def test_connect(self):
        self.conn2.modules.sys.stdout.write("hello through rpyc.ssh_connect()\n")
        self.conn2.modules.sys.stdout.flush()


if __name__ == "__main__":
    unittest.main()



from safegtk import SafeGTK
import rpyc
import gobject
import gtk
import pygtk
pygtk.require('2.0')


def BrowserServiceFactory(browser):
    class BrowserService(rpyc.Service):
        def on_connect(self, conn):
            conn._config["allow_public_attrs"] = True

        def exposed_navigate(self, url):
            old_url = browser.txt_url.get_text()
            if url.startswith("/"):
                base = old_url.split("/")[0]
                url = base + url
            browser.txt_url.set_text(url)
            browser.on_navigate(None)
    return BrowserService


class Browser(object):
    def __init__(self):
        self.conn = None

        window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        window.set_resizable(True)
        window.connect("destroy", self.on_close)
        window.set_title("Web8 Browser")
        self.box_main = gtk.VBox()
        self.box_main.show()
        window.add(self.box_main)

        hbox1 = gtk.HBox()
        hbox1.show()
        self.box_main.pack_start(hbox1, fill=False, expand=False)

        self.txt_url = gtk.Entry()
        self.txt_url.set_text("localhost/main")
        self.txt_url.show()
        self.txt_url.connect("activate", self.on_navigate)
        hbox1.pack_start(self.txt_url)

        btn_send = gtk.Button("Go")
        btn_send.show()
        btn_send.connect("clicked", self.on_navigate)
        hbox1.pack_start(btn_send, fill=False, expand=False, padding=10)

        self.box_content = None
        window.show()

    def on_close(self, widget):
        if self.conn:
            self.conn.close()
            self.conn = None
        gtk.main_quit()

    def on_navigate(self, widget, data=None):
        url = self.txt_url.get_text()
        if "/" not in url:
            url += "/"
        host, page = url.split("/", 1)
        if ":" in host:
            addr, port = host.split(":", 1)
            port = int(port)
        else:
            addr = host
            port = 18833
        if self.conn:
            self.conn.close()
            self.conn = None

        if self.box_content:
            self.box_main.remove(self.box_content)
            self.box_content.destroy()
            self.box_content = None
        self.box_content = gtk.VBox()
        self.box_content.show()
        self.box_main.pack_start(self.box_content)

        self.conn = rpyc.connect(host, port, service=BrowserServiceFactory(self))
        gobject.io_add_watch(self.conn, gobject.IO_IN, self.bg_server)
        self.conn.root.get_page(SafeGTK, self.box_content, page)

    def bg_server(self, source=None, cond=None):
        if self.conn:
            self.conn.poll_all()
            return True
        else:
            return False


if __name__ == "__main__":
    b = Browser()
    gtk.main()



import rpyc
from rpyc.utils.server import ThreadedServer
import time
import threading


class Web8Service(rpyc.Service):
    def on_connect(self, conn):
        self._conn = conn

    def exposed_get_page(self, gtk, content, page):
        self.gtk = gtk
        self.content = content
        page = page.replace(" ", "_").lower()
        pagefunc = getattr(self, f"page_{page}", None)
        if pagefunc:
            pagefunc()
        else:
            lbl1 = self.gtk.Label(f"Page {page!r} does not exist")
            lbl1.show()
            self.content.pack_start(lbl1)

    def page_main(self):
        counter = [0]

        lbl1 = self.gtk.Label("Hello mate, this is the main page")
        lbl1.show()
        self.content.pack_start(lbl1)

        def on_btn1_clicked(src):
            counter[0] += 1
            lbl2.set_text(f"You have clicked the button {counter[0]} times")

        btn1 = self.gtk.Button("Add 1")
        btn1.connect("clicked", on_btn1_clicked)
        btn1.show()
        self.content.pack_start(btn1)

        lbl2 = self.gtk.Label("You have clicked the button 0 times")
        lbl2.show()
        self.content.pack_start(lbl2)

        def on_btn2_clicked(src):
            self._conn.root.navigate("/hello_world")

        btn2 = self.gtk.Button("Go to the 'hello world' page")
        btn2.connect("clicked", on_btn2_clicked)
        btn2.show()
        self.content.pack_start(btn2)

        active = [False]

        def bg_timer_thread():
            while active[0]:
                rpyc.async_(lbl3.set_text)(f"Server time is: {time.ctime()}")
                time.sleep(1)

        bg_thread = [None]

        def on_btn3_clicked(src):
            if btn3.get_label() == "Start timer":
                bg_thread[0] = threading.Thread(target=bg_timer_thread)
                active[0] = True
                bg_thread[0].start()
                btn3.set_label("Stop timer")
            else:
                active[0] = False
                bg_thread[0].join()
                btn3.set_label("Start timer")

        btn3 = self.gtk.Button("Start timer")
        btn3.connect("clicked", on_btn3_clicked)
        btn3.show()
        self.content.pack_start(btn3)

        lbl3 = self.gtk.Label("Server time is: ?")
        lbl3.show()
        self.content.pack_start(lbl3)

    def page_hello_world(self):
        lbl = self.gtk.Label("Hello world!")
        lbl.show()
        self.content.pack_start(lbl)


if __name__ == "__main__":
    t = ThreadedServer(Web8Service, port=18833)
    t.start()



import rpyc
from rpyc.utils.server import ThreadedServer
from rpyc import SlaveService
import unittest


class Test_get_id_pack(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.port = 18878
        cls.port2 = 18879
        cls.server = ThreadedServer(SlaveService, port=cls.port, auto_register=False)
        cls.server2 = ThreadedServer(SlaveService, port=cls.port2, auto_register=False)
        cls.thd = cls.server._start_in_thread()
        cls.thd2 = cls.server2._start_in_thread()
        cls.conn = rpyc.classic.connect("localhost", port=cls.port)
        cls.conn_rpyc = cls.conn.root.getmodule('rpyc')
        cls.chained_conn = cls.conn_rpyc.connect('localhost', cls.port2)

    @classmethod
    def tearDownClass(cls):
        cls.chained_conn.close()
        cls.conn.close()
        while cls.server2.clients or cls.server.clients:
            pass  # sti
        cls.server2.close()
        cls.server.close()
        cls.thd.join()
        cls.thd2.join()

    def test_chained_connect(self):
        remote_os = self.chained_conn.root.getmodule('os')

    def test_netref(self):
        self.assertEqual(self.conn.root.____id_pack__, rpyc.lib.get_id_pack(self.conn.root))

    def test_class_instance_wo_name(self):
        ss = rpyc.SlaveService()
        id_pack = rpyc.lib.get_id_pack(ss)
        self.assertEqual('rpyc.core.service.SlaveService', id_pack[0])

    def test_class_wo_name(self):
        ss = rpyc.SlaveService
        id_pack = rpyc.lib.get_id_pack(ss)
        self.assertEqual('rpyc.core.service.SlaveService', id_pack[0])


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
import time
import signal
import multiprocessing as mp
import pdb  # noqa
import rpyc
import traceback


def echo_once():
    start = time.time()
    conn = rpyc.connect("localhost", 18861, config={"sync_request_timeout": None})
    cdelta = time.time() - start
    addr, port = conn._channel.stream.sock.getsockname()
    fileno = conn.fileno()
    start = time.time()
    conn.root.echo("Echo")
    edelta = time.time() - start
    conn.close()
    return cdelta, edelta, fileno, addr, port


def echo_forever(main_event):
    try:
        count = 0
        edelta = 0
        cdelta = 0
        _max = {'edelta': 0, 'cdelta': 0}
        fileno = "unknown"
        addr = "unknown"
        port = "unknown"
        cdelta = -1
        edelta = -1
        while main_event.is_set():
            count += 1
            cdelta, edelta, fileno, addr, port = echo_once()
            _max['cdelta'] = cdelta
            _max['edelta'] = edelta
    except KeyboardInterrupt:
        if main_event.is_set():
            main_event.clear()
    except Exception:
        tb = f"EXCEPT ('{addr}', {port}) with fd {fileno} over cdelta {cdelta} and delta {edelta}\n"
        tb += traceback.format_exc()

        return None, tb
    finally:
        return _max, None


def echo_client_pool(client_limit):
    try:
        sigint = signal.signal(signal.SIGINT, signal.SIG_IGN)
        pool = mp.Pool(processes=client_limit)
        signal.signal(signal.SIGINT, sigint)
        eid_proc = {}
        pool_manager = mp.Manager()
        main_event = pool_manager.Event()
        main_event.set()
        for eid in range(client_limit):
            eid_proc[eid] = pool.apply_async(func=echo_forever, args=(main_event,))
        while True:
            alive = len([r for r in eid_proc.values() if not r.ready()])
            print('{0}/{1} alive'.format(alive, client_limit))
            if alive == 1:
                print('All of the client processes are dead except one. Exiting loop...')
                break
            else:
                time.sleep(1)
        res = [r.get() for r in eid_proc.values() if r.ready()]
        cdelta = [_max['cdelta'] for _max, tb in res if _max]
        edelta = [_max['edelta'] for _max, tb in res if _max]
        if cdelta:
            cdelta = max(cdelta)
        else:
            cdelta = "unknown"
        if edelta:
            edelta = max(edelta)
        else:
            edelta = "unknown"
        time.sleep(1)
        print(f"Max time to establish: {cdelta}")
        print(f"Max time   echo reply: {edelta}")
        main_event.clear()
    except KeyboardInterrupt:
        main_event.clear()
        for proc in eid_proc.values():
            proc.terminate()


def main(client_limit):
    if client_limit == 1:
        echo_once()
    else:
        echo_client_pool(client_limit)


if __name__ == "__main__":
    main(client_limit=5)



from __future__ import with_statement
import rpyc
import unittest
from rpyc.lib.compat import is_py_gte37
if is_py_gte37:
    from dataclasses import dataclass

    @dataclass
    class StdTypes(object):
        exposed_intObj: int = 31
        exposed_strObj: str = "stdtypes string"
        exposed_floatObj: float = 3.14
        exposed_bytesObj: bytes = b"stdtypes bytes"
else:
    class StdTypes(object):
        pass


class MyService(rpyc.Service):

    def exposed_create_dataclass(self):
        return StdTypes()


@unittest.skipUnless(is_py_gte37, "Skipping since dataclasses is only in 3.7 and above")
class TestRemoteDataclass(unittest.TestCase):
    def setUp(self):
        self.server = rpyc.utils.server.OneShotServer(MyService, port=0)
        self.server.logger.quiet = False
        self.server._start_in_thread()
        self.conn = rpyc.connect("localhost", port=self.server.port)

    def tearDown(self):
        self.conn.close()

    def test_remote_dataclass(self):
        remote_dataclass = self.conn.root.create_dataclass()
        self.assertEqual(31, remote_dataclass.intObj)
        self.assertEqual("stdtypes string", remote_dataclass.strObj)
        self.assertEqual(3.14, remote_dataclass.floatObj)
        self.assertEqual(b"stdtypes bytes", remote_dataclass.bytesObj)


if __name__ == "__main__":
    unittest.main()



import rpyc
import copy
import unittest
from rpyc.utils.server import ThreadedServer


class MyClass(object):
    def __add__(self, other):
        return self.foo() + str(other)

    def foo(self):
        return "foo"

    def bar(self):
        return "bar"

    def spam(self):
        return "spam"

    def _privy(self):
        return "privy"

    def exposed_foobar(self):
        return "Fee Fie Foe Foo"


class YourClass(object):
    def lala(self):
        return MyClass()

    def baba(self):
        return "baba"

    def gaga(self):
        return "gaga"


try:
    long
except NameError:
    long = int
    unicode = str

try:
    bytes
except NameError:
    bytes = str


class Protector(object):
    def __init__(self, safetypes=(int, list, bool, tuple, str, float, long, unicode, bytes)):
        self._safetypes = set(safetypes)
        self._typereg = {}

    def register(self, typ, attrs):
        self._typereg[typ] = frozenset(attrs)

    def wrap(self, obj):
        class Restrictor(object):
            def __call__(_, *args, **kwargs):
                return self.wrap(obj(*args, **kwargs))

            def _rpyc_getattr(_, name):
                if type(obj) not in self._safetypes:
                    attrs = self._typereg.get(type(obj), ())
                    if name not in attrs:
                        raise AttributeError(name)
                obj2 = getattr(obj, name)
                return self.wrap(obj2)
            __getattr__ = _rpyc_getattr
        return Restrictor()


SVC_RESTRICTED = ["exposed_foobar", "__add__", "_privy", "foo", "bar"]


class MyService(rpyc.Service):
    exposed_MyClass = MyClass

    def exposed_get_one(self):
        return rpyc.restricted(MyClass(), SVC_RESTRICTED)

    def exposed_get_two(self):
        protector = Protector()
        protector.register(MyClass, SVC_RESTRICTED)
        protector.register(YourClass, ["lala", "baba"])
        return protector.wrap(YourClass())


class TestRestricted(unittest.TestCase):
    def setUp(self):
        self.server = ThreadedServer(MyService)
        self.thd = self.server._start_in_thread()
        self.conn = rpyc.connect("localhost", self.server.port)

    def tearDown(self):
        self.conn.close()
        while self.server.clients:
            pass
        self.server.close()
        self.thd.join()

    def test_restricted(self):
        obj = self.conn.root.get_one()
        self.assertEqual(obj.foo(), "foo")
        self.assertEqual(obj.bar(), "bar")
        self.assertEqual(obj.__add__("bar"), "foobar")
        self.assertEqual(obj._privy(), "privy")
        self.assertEqual(obj.exposed_foobar(), "Fee Fie Foe Foo")
        self.assertRaises(AttributeError, lambda: obj.spam)

    def test_restricted2(self):
        self.server.protocol_config = {'allow_public_attrs': False}
        obj = self.conn.root.get_one()
        self.assertEqual(obj.foo(), "foo")
        self.assertEqual(obj.bar(), "bar")
        self.assertEqual(obj.__add__("bar"), "foobar")
        self.assertEqual(obj._privy(), "privy")
        self.assertRaises(AttributeError, lambda: obj.spam)


class TestConfigAllows(unittest.TestCase):
    def setUp(self):
        self.cfg = self._reset_cfg()
        self.server = ThreadedServer(MyService, port=0)
        self.thd = self.server._start_in_thread()
        self.conn = rpyc.connect("localhost", self.server.port)

    def tearDown(self):
        self.conn.close()
        while self.server.clients:
            pass
        self.server.close()
        self.thd.join()

    def _reset_cfg(self):
        self.cfg = copy.copy(rpyc.core.protocol.DEFAULT_CONFIG)
        return self.cfg

    def _get_myclass(self, proto_config):
        self.conn.close()
        self.server.protocol_config.update(proto_config)
        self.conn = rpyc.connect("localhost", self.server.port)
        return self.conn.root.MyClass()

    def test_default_config(self):
        obj = self._get_myclass(self.cfg)
        self.assertEqual(obj + 'bar', "foobar")
        self.assertEqual(obj.foobar(), "Fee Fie Foe Foo")
        self.assertEqual(obj.exposed_foobar(), "Fee Fie Foe Foo")
        self.assertRaises(AttributeError, lambda: obj._privy)
        self.assertRaises(AttributeError, lambda: obj.foo)
        self.assertRaises(AttributeError, lambda: obj.bar)
        self.assertRaises(AttributeError, lambda: obj.spam)

    def test_allow_all(self):
        self._reset_cfg()
        self.cfg['allow_all_attrs'] = True
        obj = self._get_myclass(self.cfg)
        self.assertEqual(obj + 'bar', "foobar")
        self.assertEqual(obj.__add__("bar"), "foobar")
        self.assertEqual(obj._privy(), "privy")
        self.assertEqual(obj.foobar(), "Fee Fie Foe Foo")
        self.assertEqual(obj.exposed_foobar(), "Fee Fie Foe Foo")

    def test_allow_exposed(self):
        self._reset_cfg()
        self.cfg['allow_exposed_attrs'] = False
        try:
            self._get_myclass(self.cfg)  # returns obj, but ignored
            passed = False
        except Exception:
            passed = True
        self.assertEqual(passed, True)

    def test_allow_safe_attrs(self):
        self._reset_cfg()
        self.cfg['allow_safe_attrs'] = False
        obj = self._get_myclass(self.cfg)
        self.assertEqual(obj.foobar(), "Fee Fie Foe Foo")
        self.assertEqual(obj.exposed_foobar(), "Fee Fie Foe Foo")
        self.assertRaises(AttributeError, lambda: obj._privy)
        self.assertRaises(AttributeError, lambda: obj + 'bar')
        self.assertRaises(AttributeError, lambda: obj.foo)
        self.assertRaises(AttributeError, lambda: obj.bar)
        self.assertRaises(AttributeError, lambda: obj.spam)

    def test_allow_public_attrs(self):
        self._reset_cfg()
        self.cfg['allow_public_attrs'] = True
        obj = self._get_myclass(self.cfg)
        self.assertEqual(obj + 'bar', "foobar")
        self.assertEqual(obj.foo(), "foo")
        self.assertEqual(obj.bar(), "bar")
        self.assertEqual(obj.foobar(), "Fee Fie Foe Foo")
        self.assertEqual(obj.exposed_foobar(), "Fee Fie Foe Foo")
        self.assertRaises(AttributeError, lambda: obj._privy)


class MyDescriptor1(object):
    def __get__(self, instance, owner=None):
        raise AttributeError("abcd")


class MyDescriptor2(object):
    def __get__(self, instance, owner=None):
        if instance is None:
            return self
        else:
            raise RuntimeError("efgh")


@rpyc.service
class MyDecoratedService(rpyc.Service):
    desc_1 = rpyc.exposed(MyDescriptor1())
    exposed_desc_2 = MyDescriptor2()


class TestDescriptorErrors(unittest.TestCase):
    """Validate stack traces are consistent independent of how exposed attribute is accessed #478 #479"""

    def setUp(self):
        self.cfg = copy.copy(rpyc.core.protocol.DEFAULT_CONFIG)
        self.server = ThreadedServer(MyDecoratedService(), port=0)
        self.thd = self.server._start_in_thread()
        self.conn = rpyc.connect("localhost", self.server.port)

    def tearDown(self):
        self.conn.close()
        while self.server.clients:
            pass
        self.server.close()
        self.thd.join()

    def test_default_config(self):
        root = self.conn.root
        self.assertRaisesRegex(AttributeError, "abcd", lambda: root.exposed_desc_1)
        self.assertRaisesRegex(AttributeError, "abcd", lambda: root.desc_1)
        self.assertRaisesRegex(RuntimeError, "efgh", lambda: root.exposed_desc_2)
        self.assertRaisesRegex(RuntimeError, "efgh", lambda: root.desc_2)

    def test_allow_all(self):
        self.cfg['allow_all_attrs'] = True
        self.conn.close()
        self.server.protocol_config.update(self.cfg)
        self.conn = rpyc.connect("localhost", self.server.port)
        root = self.conn.root
        self.assertRaisesRegex(AttributeError, "abcd", lambda: root.exposed_desc_1)
        self.assertRaisesRegex(AttributeError, "abcd", lambda: root.desc_1)
        self.assertRaisesRegex(RuntimeError, "efgh", lambda: root.exposed_desc_2)
        self.assertRaisesRegex(RuntimeError, "efgh", lambda: root.desc_2)


if __name__ == "__main__":
    unittest.main()



from __future__ import with_statement
import rpyc
import unittest


class MyService(rpyc.Service):

    def exposed_set_version(self):
        rpyc.version.__version__ = '1.0.0'

    def exposed_remote_assert(self, val):
        assert val


class TestRemoteException(unittest.TestCase):
    def setUp(self):
        self.server = rpyc.utils.server.OneShotServer(MyService, port=0)
        self.server.logger.quiet = False
        self.server._start_in_thread()
        self.original_version_string = rpyc.version.__version__
        self.conn = rpyc.connect("localhost", port=self.server.port)

    def tearDown(self):
        rpyc.version.__version__ = self.original_version_string
        self.conn.close()

    def test_remote_exception(self):
        # Since the server/client share the same namespace, the version will change for both.
        # Even so, this should suffice for unit testing
        warn_msg = 'WARNING: Remote is on RPyC 1.0.0 and local is on RPyC 1.0.0.'
        try:
            self.conn.root.remote_assert(False)
        except Exception as exc:
            exc_rpyc_version = exc._remote_version
            exc_remote_tb = exc._remote_tb
        else:
            exc_rpyc_version = None
            exc_remote_tb = ''
        self.assertEqual(self.original_version_string, exc_rpyc_version)
        self.assertFalse(warn_msg in exc_remote_tb)
        try:
            self.conn.root.set_version()
            self.conn.root.remote_assert(False)
        except Exception as exc:
            exc_rpyc_version = exc._remote_version
            exc_remote_tb = exc._remote_tb
        else:
            exc_rpyc_version = None
            exc_remote_tb = ''
        self.assertEqual('1.0.0', exc_rpyc_version)
        self.assertTrue(warn_msg in exc_remote_tb)


class TestExclusionsRemoteException(unittest.TestCase):
    def setUp(self):
        config = {'include_local_traceback': False, 'include_local_version': False}
        self.server = rpyc.utils.server.OneShotServer(MyService, port=0, protocol_config=config)
        self.server.logger.quiet = False
        self.server._start_in_thread()
        self.conn = rpyc.connect("localhost", port=self.server.port)

    def tearDown(self):
        self.conn.close()

    def test_remote_exception(self):
        try:
            self.conn.root.remote_assert(False)
        except Exception as exc:
            exc_rpyc_version = exc._remote_version
            exc_remote_tb = exc._remote_tb
        else:
            exc_rpyc_version = None
            exc_remote_tb = ''
        self.assertEqual("<traceback denied>", exc_remote_tb)
        self.assertEqual("<version denied>", exc_rpyc_version)


if __name__ == "__main__":
    unittest.main()



from __future__ import print_function
import sys
import pickle  # noqa
import timeit
import rpyc
import unittest
from tests import cfg_tests
try:
    import pandas as pd
    import numpy as np
    _pampy_import_failed = False
except Exception:
    pd = None
    np = None
    _pampy_import_failed = True


DF_ROWS = 2000
DF_COLS = 2500


class MyService(rpyc.Service):
    on_connect_called = False
    on_disconnect_called = False

    def on_connect(self, conn):
        self.on_connect_called = True

    def on_disconnect(self, conn):
        self.on_disconnect_called = True

    def exposed_write_data(self, dataframe):
        rpyc.classic.obtain(dataframe)

    def exposed_get(self):
        return np.random.rand(3, 3)

    def exposed_ping(self):
        return "pong"


@unittest.skipIf(_pampy_import_failed, "Pandas & numpy are not available")
class TestServicePickle(unittest.TestCase):
    """Issues #323 and #329 showed for large objects there is an excessive number of round trips.

    This test case should check the interrelations of
        + MAX_IO_CHUNK
        + min twrite
        + occurrence rate of socket timeout for other clients
    """
    config = {}

    def setUp(self):
        self.cfg = {'allow_pickle': True}
        self.server = rpyc.utils.server.ThreadedServer(MyService, port=0, protocol_config=self.cfg.copy())
        self.server.logger.quiet = False
        self.thd = self.server._start_in_thread()
        self.conn = rpyc.connect("localhost", self.server.port, config=self.cfg)
        self.conn2 = rpyc.connect("localhost", self.server.port, config=self.cfg)
        # globals are made available to timeit, prepare them
        cfg_tests.timeit['conn'] = self.conn
        cfg_tests.timeit['conn2'] = self.conn2
        cfg_tests.timeit['df'] = pd.DataFrame(np.random.rand(DF_ROWS, DF_COLS))

    def tearDown(self):
        self.conn.close()
        self.server.close()
        self.thd.join()
        cfg_tests.timeit.clear()

    def test_dataframe_pickling(self):
        # the proxy will sync w/ the pickle handle and default proto and provide this as the argument to pickle.load
        # By timing how long w/ out any round trips pickle.dumps and picke.loads takes, the overhead of RPyC protocol
        # can be found

        rpyc.core.channel.Channel.COMPRESSION_LEVEL = 1
        # rpyc.core.stream.SocketStream.MAX_IO_CHUNK = 8000
        level = rpyc.core.channel.Channel.COMPRESSION_LEVEL
        max_chunk = rpyc.core.stream.SocketStream.MAX_IO_CHUNK
        repeat = 3
        number = 1
        pickle_stmt = 'pickle.loads(pickle.dumps(cfg_tests.timeit["df"]))'
        write_stmt = 'rpyc.lib.spawn(cfg_tests.timeit["conn"].root.write_data, cfg_tests.timeit["df"]); '
        write_stmt += '[cfg_tests.timeit["conn2"].root.ping() for i in range(30)]'
        # write_stmt = 'cfg_tests.timeit["conn"].root.write_data(cfg_tests.timeit["df"])'
        t = timeit.Timer(pickle_stmt, globals=globals())
        tpickle = min(t.repeat(repeat, number))
        t = timeit.Timer(write_stmt, globals=globals())
        twrite = min(t.repeat(repeat, number))

        headers = ['sample', 'tpickle', 'twrite', 'bytes', 'level', 'max_chunk']  # noqa
        data = [repeat, tpickle, twrite, sys.getsizeof(cfg_tests.timeit['df']), level, max_chunk]
        data = [str(d) for d in data]
        # print(','.join(headers), file=open('/tmp/time.csv', 'a'))
        # print(','.join(data), file=open('/tmp/time.csv', 'a'))


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
import time
import unittest
import rpyc


class MyService(rpyc.Service):
    class exposed_Invoker(object):
        def __init__(self, callback, interval):
            self.callback = rpyc.async_(callback)
            self.interval = interval
            self.active = True
            self.thread = rpyc.spawn(self.work)

        def exposed_stop(self):
            self.active = False
            self.thread.join()

        def work(self):
            while self.active:
                self.callback(time.time())
                time.sleep(self.interval)

    def exposed_foo(self, x):
        time.sleep(2)
        return x * 17


class Test_Multithreaded(unittest.TestCase):
    def setUp(self):
        self.conn = rpyc.connect_thread(remote_service=MyService)
        self.bgserver = rpyc.BgServingThread(self.conn)

    def tearDown(self):
        self.bgserver.stop()
        self.conn.close()

    def test_invoker(self):
        counter = [0]

        def callback(x):
            counter[0] += 1
            print(f"callback {x}")
        invoker = self.conn.root.Invoker(callback, 1)
        # 3 * 2sec = 6 sec = ~6 calls to callback
        for i in range(3):
            print(f"foo{i} = {self.conn.root.foo(i)}")
        invoker.stop()
        print(f"callback called {counter[0]} times")
        self.assertTrue(counter[0] >= 5)


if __name__ == "__main__":
    unittest.main()



"""
this class exposes only the gtk constants and some of the "safe" classes.
we don't want the server to open pop-ups on the client, so we won't expose
Window() et al.
"""
import gtk
import pygtk
pygtk.require('2.0')


safe_gtk_classes = set([
    "Box", "VBox", "HBox", "Frame", "Entry", "Button", "ScrolledWindow",
    "TextView", "Label",
])


class SafeGTK(object):
    for _name in dir(gtk):
        if _name in safe_gtk_classes or _name.isupper():
            exec(f"exposed_{_name} = gtk.{_name}")
    del _name


SafeGTK = SafeGTK()



import inspect
import math
import rpyc
from rpyc.utils.server import ThreadedServer
from rpyc import SlaveService
from rpyc.core import netref
import unittest


logger = rpyc.lib.setup_logger()


class MyMeta(type):

    def spam(self):
        return self.__name__ * 5


class MyClass(object):
    __metaclass__ = MyMeta


class MyService(rpyc.Service):
    on_connect_called = False
    on_disconnect_called = False

    def on_connect(self, conn):
        self.on_connect_called = True

    def on_disconnect(self, conn):
        self.on_disconnect_called = True

    def exposed_distance(self, p1, p2):
        x1, y1 = p1
        x2, y2 = p2
        return math.sqrt((x2 - x1) ** 2 + (y2 - y1) ** 2)

    def exposed_getlist(self):
        return [
            1, 2, 3]

    def foobar(self):
        assert False

    def exposed_getmeta(self):
        return MyClass()

    def exposed_instance(self, inst, cls):
        return isinstance(inst, cls)

    def exposed_getnonetype(self):
        """ About the unit test - what's common to types.MethodType and NoneType is that both are
        not accessible via builtins. So the unit test I've added in 108ff8e was enough to
        my understanding (implement it with NoneType because that's more easily "created") """
        return type(None)


class TestBaseNetrefMRO(unittest.TestCase):
    def setUp(self):
        self.conn = rpyc.classic.connect_thread()

    def tearDown(self):
        self.conn.close()
        self.conn = None

    def test_mro(self):
        # TODO: netref.class_factory, redesign to register builtin types and better handle generic-aliases/types
        #   - components to explore: abc.ABCMeta, abc.ABC.register types
        #   - add mro test for netrefs to remote builtins
        self.assertEqual(netref.NetrefMetaclass.__mro__, (netref.NetrefMetaclass, type, object))

    def test_basenetref(self):
        self.assertIsInstance(netref.BaseNetref, netref.NetrefMetaclass)
        self.assertIsInstance(netref.BaseNetref, object)
        mro = inspect.getmro(netref.BaseNetref)
        self.assertEqual(mro, (netref.BaseNetref, object))

    def test_builtins_dict_netref(self):
        cls = netref.builtin_classes_cache['builtins.dict']
        mro_netref = inspect.getmro(cls)
        mro_dict = inspect.getmro(dict)
        logger.debug('\n')
        logger.debug(f'dict_netref: {mro_netref}')
        logger.debug(f'dict:        {mro_dict}')
        self.conn.execute("dict_ = dict(a=0xd35db33f)")
        remote_dict = self.conn.namespace['dict_']
        logger.debug(f'remote_dict: {remote_dict}')
        self.assertEqual(remote_dict['a'], 3546133311)


class Test_Netref_Hierarchy(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.server = ThreadedServer(SlaveService, port=18878, auto_register=False)
        cls.server.logger.quiet = False
        cls.server._start_in_thread()

    def setUp(self):
        self.conn = rpyc.classic.connect('localhost', port=18878)

    @classmethod
    def tearDownClass(cls):
        cls.server.close()

    def tearDown(self):
        self.conn.close()

    def test_instancecheck_across_connections(self):
        self.conn2 = rpyc.classic.connect('localhost', port=18878)
        self.conn.execute('import tests.test_magic')
        self.conn2.execute('import tests.test_magic')
        foo = self.conn.modules.tests.test_magic.Foo()
        bar = self.conn.modules.tests.test_magic.Bar()
        self.assertTrue(isinstance(foo, self.conn.modules.tests.test_magic.Foo))
        self.assertTrue(isinstance(bar, self.conn2.modules.tests.test_magic.Bar))
        self.assertFalse(isinstance(bar, self.conn.modules.tests.test_magic.Foo))
        with self.assertRaises(TypeError):
            isinstance(self.conn.modules.tests.test_magic.Foo, bar)

    def test_classic(self):
        x = self.conn.builtin.list((1, 2, 3, 4))
        self.assertTrue(isinstance(x, list))
        self.assertTrue(isinstance(x, rpyc.BaseNetref))
        with self.assertRaises(TypeError):
            isinstance([], x)
        i = 0
        self.assertTrue(type(x).__getitem__(x, i) == x.__getitem__(i))

    def test_builtins(self):
        _builtins = self.conn.modules.builtins
        self.assertEqual(repr(_builtins.dict), repr(dict))  # Check repr behavior of netref matches local
        self.assertEqual(repr(type(_builtins.dict.__class__)), repr(type))  # Check netref __class__ is type
        self.assertIs(type(_builtins.dict.__class__), type)
        # Check class descriptor for netrefs
        dict_ = _builtins.dict(space='remote')
        self.assertIs(type(dict_).__dict__['__class__'].instance, dict)
        self.assertIs(type(dict_).__dict__['__class__'].owner, type)

    def test_instancecheck_list(self):
        service = MyService()
        conn = rpyc.connect_thread(remote_service=service)
        conn.root
        remote_list = conn.root.getlist()
        self.assertTrue(conn.root.instance(remote_list, list))
        conn.close()

    def test_instancecheck_none(self):
        """
        test for the regression reported in https://github.com/tomerfiliba-org/rpyc/issues/426
        """
        service = MyService()
        conn = rpyc.connect_thread(remote_service=service)
        remote_NoneType = conn.root.getnonetype()
        self.assertTrue(isinstance(None, remote_NoneType))
        conn.close()

    def test_StandardError(self):
        _builtins = self.conn.modules.builtins if rpyc.lib.compat.is_py_3k else self.conn.modules.__builtin__
        self.assertTrue(isinstance(_builtins.Exception(), _builtins.BaseException))
        self.assertTrue(isinstance(_builtins.Exception(), _builtins.Exception))
        self.assertTrue(isinstance(_builtins.Exception(), BaseException))
        self.assertTrue(isinstance(_builtins.Exception(), Exception))

    def test_modules(self):
        """
        >>> type(unittest)
        <class 'module'>
        >>> type(self.conn.modules.unittest)
        <netref class 'rpyc.core.netref.unittest'>  # reflects that it is a proxy object to unittest
        >>> unittest.__class__
        <class 'module'>  # base case
        >>> conn.modules.unittest.__class__
        <class 'module'>  # matches base case
        >>> type(unittest.__class__)
        <class 'type'>  # base case
        >>> type(conn.modules.unittest.__class__)
        <class 'type'>  # matches base case
        """
        # instance module assertions
        self.assertEqual(repr(self.conn.modules.unittest), repr(unittest))
        self.assertEqual(repr(type(self.conn.modules.unittest)), "<netref class 'rpyc.core.netref.unittest'>")
        self.assertIs(self.conn.modules.unittest.__class__, type(unittest))
        self.assertIs(type(self.conn.modules.unittest.__class__), type)
        # class module assertions
        remote_module_cls = self.conn.modules.builtins.type(self.conn.modules.sys)
        remote_module_cls_id = self.conn.modules.builtins.id(remote_module_cls)
        self.assertEqual(repr(remote_module_cls), "<class 'module'>")
        self.assertEqual(remote_module_cls.____id_pack__, ('builtins.module', remote_module_cls_id, 0))


    def test_proxy_instancecheck(self):
        self.assertIsInstance(self.conn.modules.builtins.RuntimeError(), Exception)
        # TODO: below should pass
        # self.assertIsInstance(self.conn.modules.builtins.RuntimeError(), self.conn.modules.builtins.Exception)


if __name__ == '__main__':
    unittest.main()



import sys
import time
import unittest
from tests import support
import rpyc


class Test_Affinity(unittest.TestCase):
    """To find race conditions we vary processor affinity (CPU pinning) settings.

    GIL tends to context switch more frequently when more CPU cores are available. By running binding this PID
    to one CPU core, more ticks will occur between each context switch. Increasing the number of CPU cores we are bound to
    will run be able to test RPyC with more frequent context switching. The aim is to find contention between threads for
    the socket that result in undesired behavior (e.g. a reply never making it to the right thread).

    Python Thread Visualization: http://www.dabeaz.com/GIL/gilvis/fourthread.html
    """
    @classmethod
    def setUpClass(cls):
        """Construct the a copy of ClassicServer that embeds a sleep(0) into _dispatch and set affinity"""
        cls._orig_func = rpyc.core.protocol.Connection._dispatch

        def _sleepy_dispatch(self, data):
            time.sleep(0.0)
            return cls._orig_func(self, data)
        setattr(rpyc.core.protocol.Connection, '_dispatch', _sleepy_dispatch)
        cls.cfg = {'sync_request_timeout': 5}
        if sys.platform != "linux":
            print("Running Test_Affinity is less productive on non-linux systems...")
        try:
            cls._skip = None
            cls._os = None
            cls._supported = True
            cls._os = support.import_module('os', fromlist=('sched_setaffinity', 'sched_getaffinity'))
            cls._orig_affinity = cls._os.sched_getaffinity(0)
        except unittest.SkipTest as skip:
            cls._skip = skip
            cls._supported = False
            cls._orig_affinity = None

    @classmethod
    def tearDownClass(cls):
        setattr(rpyc.core.protocol.Connection, '_dispatch', cls._orig_func)

    def setUp(self):
        self._os.sched_setaffinity(0, {0, })
        self.conn = rpyc.connect_thread(rpyc.ClassicService, self.cfg, rpyc.ClassicService, self.cfg)
        self.bg_threads = [rpyc.BgServingThread(self.conn) for i in range(3)]

    def tearDown(self):
        for t in self.bg_threads:
            t.stop()
        self.bg_threads = []
        self.conn.close()
        self.conn = None
        self._reset_affinity()

    def _time_execute_sleep(self):
        """returns time to execute 0.3s worth of sleeping"""
        t0 = time.time()
        self.conn.execute("import time")
        for p in (0, 0.1, 0.2):
            self.conn.execute(f"time.sleep({p})")
        return time.time() - t0

    def _reset_affinity(self):
        if self._os is not None:
            return self._os.sched_setaffinity(0, self._orig_affinity)

    def test_pinned_to_0(self):
        """test behavior with processor affinity set such that this process is pinned to 0"""
        if self._skip:
            raise self._skip
        max_elapsed_time = self.cfg['sync_request_timeout']
        elapsed_time = self._time_execute_sleep()
        self.assertLess(elapsed_time, max_elapsed_time)
        self.assertIn('count=0', repr(self.conn._recvlock))



#!/usr/bin/env python3
import inspect
import logging
# import gevent
# from gevent import monkey
# monkey.patch_all()
import rpyc


class EchoService(rpyc.Service):
    def on_connect(self, conn):
        msg = f"on connect service peer name: {conn._channel.stream.sock.getpeername()}"
        conn._config["logger"].debug(msg)

    def on_disconnect(self, conn):
        pass

    def exposed_echo(self, message):
        if message == "Echo":
            return "Echo Reply"
        else:
            return "Parameter Problem"


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    echo_svc = rpyc.OneShotServer(service=EchoService, port=18861, protocol_config={'allow_all_attrs': True})
    echo_svc.start()



import rpyc
import time
import os


filename = "/tmp/floop.bloop"
if os.path.exists(filename):
    os.remove(filename)

f = open(filename, "w")
conn = rpyc.connect("localhost", 18871)
bgsrv = rpyc.BgServingThread(conn)  # create a bg thread to process incoming events


def on_file_changed(oldstat, newstat):
    print("file changed")
    print(f"    old stat: {oldstat}")
    print(f"    new stat: {newstat}")


mon = conn.root.FileMonitor(filename, on_file_changed)  # create a filemon

print("wait a little for the filemon to have a look at the original file")
time.sleep(2)

print("change file size")
f.write("shmoop")  # change size
f.flush()
time.sleep(2)

print("change size again")
f.write("groop")  # change size
f.flush()
time.sleep(2)

mon.stop()
bgsrv.stop()
conn.close()



import rpyc
import os
import unittest
from rpyc.utils.authenticators import SSLAuthenticator
from rpyc.utils.server import ThreadedServer
from rpyc import SlaveService

try:
    import ssl  # noqa
    _ssl_import_failed = False
except ImportError:
    _ssl_import_failed = True


@unittest.skipIf(_ssl_import_failed, "Ssl not available")
class Test_SSL(unittest.TestCase):
    '''
    created key like that
    http://www.akadia.com/services/ssh_test_certificate.html

    openssl req -newkey rsa:1024 -nodes -keyout mycert.pem -out mycert.pem
    '''

    def setUp(self):
        self.key = os.path.join(os.path.dirname(__file__), "server.key")
        self.cert = os.path.join(os.path.dirname(__file__), "server.crt")
        print(self.cert, self.key)

        authenticator = SSLAuthenticator(self.key, self.cert)
        self.server = ThreadedServer(SlaveService, port=18812,
                                     auto_register=False, authenticator=authenticator)
        self.server.logger.quiet = False
        self.server._start_in_thread()

    def tearDown(self):
        while self.server.clients:
            pass
        self.server.close()

    def test_ssl_conenction(self):
        c = rpyc.classic.ssl_connect("localhost", port=18812,
                                     keyfile=self.key, certfile=self.cert)
        print(repr(c))
        print(c.modules.sys)
        print(c.modules["xml.dom.minidom"].parseString("<a/>"))
        c.execute("x = 5")
        self.assertEqual(c.namespace["x"], 5)
        self.assertEqual(c.eval("1+x"), 6)
        c.close()


if __name__ == "__main__":
    unittest.main()



import gdb
import rpyc


class GDBService(rpyc.Service):
    def exposed_get(self):
        return gdb

    def exposed_quit(self):
        gdb.execute('quit')


if __name__ == "__main__":
    server = rpyc.ThreadedServer(GDBService, port=0, protocol_config={'allow_all_attrs': True})
    gdb.write('{}\n'.format(server.port))
    gdb.flush()
    server.start()



#!/usr/bin/env python
"""Emulates a service function that is blocked due to being busy/sleeping.

Additional context: https://github.com/tomerfiliba-org/rpyc/issues/491#issuecomment-1131843406
"""
import logging
import time
import rpyc
import threading


logger = rpyc.setup_logger(namespace='server')
rpyc.core.protocol.DEFAULT_CONFIG['logger'] = logger


class Service(rpyc.Service):
    def exposed_fetch_value(self):
        return self._value

    def exposed_function(self, client_event, block_server_thread=False):
        if block_server_thread:
            # For some reason
            def _wait(): return getattr(client_event, 'wait')()  # delays attr proxy behavior
            def _set(): return getattr(client_event, 'set')()  # delays attr proxy behavior
        else:
            _wait = rpyc.async_(client_event.wait)  # amortize proxy behavior
            _set = rpyc.async_(client_event.set)  # amortize proxy behavior
        _wait()
        logger.debug('Client messaged to wait for now...')
        for i in (1, 2):
            logger.debug(f'Pretending to do task {i}')
            time.sleep(0.2)
        self._value = 6465616462656566  # ''.join([hex(ord(c))[2:] for c in 'deadbeef'])
        _set()
        logger.debug('Client event set, it may resume...')


if __name__ == "__main__":
    rpyc.ThreadedServer(service=Service, hostname="localhost", port=18812).start()



import time
import unittest
import rpyc


class TestAsync(unittest.TestCase):
    def setUp(self):
        self.conn = rpyc.classic.connect_thread()
        self.a_sleep = rpyc.async_(self.conn.modules.time.sleep)
        self.a_int = rpyc.async_(self.conn.builtin.int)

    def tearDown(self):
        self.conn.close()

    def test_asyncresult_api(self):
        res = self.a_sleep(2)
        self.assertFalse(res.ready)
        res.wait()
        self.assertTrue(res.ready)
        self.assertFalse(res.expired)
        self.assertFalse(res.error)
        self.assertEqual(res.value, None)

    def test_asyncresult_expiry(self):
        res = self.a_sleep(5)
        res.set_expiry(4)
        t0 = time.time()
        self.assertRaises(rpyc.AsyncResultTimeout, res.wait)
        dt = time.time() - t0
        # print(f"timed out after {dt}")
        self.assertTrue(dt >= 3.5, str(dt))
        self.assertTrue(dt <= 4.5, str(dt))

    def test_asyncresult_callbacks(self):
        res = self.a_sleep(2)
        visited = []

        def f(res):
            assert res.ready
            assert not res.error
            visited.append("f")

        def g(res):
            visited.append("g")

        res.add_callback(f)
        res.add_callback(g)
        res.wait()
        self.assertEqual(set(visited), set(["f", "g"]))

    def test_timed(self):
        timed_sleep = rpyc.timed(self.conn.modules.time.sleep, 5)
        res = timed_sleep(3)
        res.value
        res = timed_sleep(7)
        self.assertRaises(rpyc.AsyncResultTimeout, lambda: res.value)

    def test_exceptions(self):
        res = self.a_int("foo")
        res.wait()
        self.assertTrue(res.error)
        self.assertRaises(ValueError, lambda: res.value)


if __name__ == "__main__":
    unittest.main()



import rpyc
import unittest
from rpyc.utils.server import ThreadedServer
from rpyc import SlaveService


# travis: "Network is unreachable", https://travis-ci.org/tomerfiliba/rpyc/jobs/108231239#L450
@unittest.skip("requires IPv6")
class Test_IPv6(unittest.TestCase):
    def setUp(self):
        self.server = ThreadedServer(SlaveService, port=0, ipv6=True)
        self.server.logger.quiet = True
        self.thd = self.server._start_in_thread()

    def tearDown(self):
        self.server.close()
        self.thd.join()

    def test_ipv6_conenction(self):
        c = rpyc.classic.connect("::1", port=self.server.port, ipv6=True)
        print(repr(c))
        print(c.modules.sys)
        print(c.modules["xml.dom.minidom"].parseString("<a/>"))
        c.execute("x = 5")
        self.assertEqual(c.namespace["x"], 5)
        self.assertEqual(c.eval("1+x"), 6)
        c.close()


if __name__ == "__main__":
    unittest.main()



from __future__ import with_statement

import unittest
import subprocess
import sys

from plumbum import SshMachine
from plumbum.machines.paramiko_machine import ParamikoMachine
from rpyc.utils.zerodeploy import DeployedServer
from rpyc.core import DEFAULT_CONFIG
try:
    import paramiko  # noqa
    _paramiko_import_failed = False
except Exception:
    _paramiko_import_failed = True


class TestDeploy(unittest.TestCase):
    def test_deploy(self):
        rem = SshMachine("localhost")
        rem.env['RPYC_BIND_THREADS'] = str(DEFAULT_CONFIG['bind_threads']).lower()
        SshMachine.python = rem[sys.executable]
        with DeployedServer(rem) as dep:
            conn = dep.classic_connect()
            print(conn.modules.sys)
            func = conn.modules.os.getcwd
            print(func())
            conn.close()

        try:
            func()
        except EOFError:
            pass
        else:
            self.fail("expected an EOFError")
        rem.close()

    def test_close_timeout(self):
        expected_timeout = 4
        observed_timeouts = []
        original_communicate = subprocess.Popen.communicate

        def replacement_communicate(self, input=None, timeout=None):
            observed_timeouts.append(timeout)
            return original_communicate(self, input, timeout)

        try:
            subprocess.Popen.communicate = replacement_communicate
            rem = SshMachine("localhost")
            rem.env['RPYC_BIND_THREADS'] = str(DEFAULT_CONFIG['bind_threads']).lower()
            SshMachine.python = rem[sys.executable]
            dep = DeployedServer(rem)
            conn = dep.classic_connect()
            dep.close(timeout=expected_timeout)
            rem.close()
            conn.close()
        finally:
            subprocess.Popen.communicate = original_communicate
        # The last three calls to communicate() happen during close(), so check they
        # applied the timeout.
        self.assertEqual(observed_timeouts[-3:], [expected_timeout] * 3)

    def test_close_timeout_default_none(self):
        observed_timeouts = []
        original_communicate = subprocess.Popen.communicate

        def replacement_communicate(self, input=None, timeout=None):
            observed_timeouts.append(timeout)
            return original_communicate(self, input, timeout)

        try:
            subprocess.Popen.communicate = replacement_communicate
            rem = SshMachine("localhost")
            rem.env['RPYC_BIND_THREADS'] = str(DEFAULT_CONFIG['bind_threads']).lower()
            SshMachine.python = rem[sys.executable]
            dep = DeployedServer(rem)
            conn = dep.classic_connect()
            dep.close()
            rem.close()
            conn.close()
        finally:
            subprocess.Popen.communicate = original_communicate
        # No timeout specified, so Popen.communicate should have been called with timeout None.
        self.assertEqual(observed_timeouts, [None] * len(observed_timeouts))

    @unittest.skipIf(_paramiko_import_failed, "Paramiko is not available")
    def test_deploy_paramiko(self):
        rem = ParamikoMachine("localhost", missing_host_policy=paramiko.AutoAddPolicy())
        rem.env['RPYC_BIND_THREADS'] = str(DEFAULT_CONFIG['bind_threads']).lower()
        with DeployedServer(rem) as dep:
            conn = dep.classic_connect()
            print(conn.modules.sys)
            func = conn.modules.os.getcwd
            print(func())
            conn.close()

        try:
            func()
        except EOFError:
            pass
        else:
            self.fail("expected an EOFError")
        rem.close()


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
import logging
import functools
import rpyc
import threading
import random
import time


THREAD_SAFE = True  # Toggles thread safe and unsafe behavior


def synchronize(lock):
    """ Decorator that invokes the lock acquire call before a function call and releases after """
    def sync_func(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            lock.acquire()
            res = func(*args, **kwargs)
            lock.release()
            return res
        return wrapper
    return sync_func


class SharingComponent(object):
    """ Initialized in the class definition of SharingService and shared by all instances of SharingService """
    lock = threading.Lock()

    def __init__(self):
        self.sequence_id = 0

    def sleepy_sequence_id(self):
        """ increment id and sometimes sleep to force race condition """
        self.sequence_id += 1
        _expected_sequence_id = self.sequence_id
        if random.randint(0, 1) == 1:
            time.sleep(1)
        if self.sequence_id == _expected_sequence_id:
            return self.sequence_id
        else:
            raise RuntimeError("Unexpected sequence_id behavior (race condition).")

    @synchronize(lock)
    def get_sequence_id(self):
        """ provides a thread-safe execution frame to otherwise unsafe functions """
        return self.sleepy_sequence_id()


class SharingService(rpyc.Service):
    """ A class that allows for sharing components between connection instances """
    __shared__ = SharingComponent()

    @property
    def shared(self):
        """ convenient access to an otherwise long object name """
        return SharingService.__shared__

    def exposed_echo(self, message):
        """ example of the potential perils when threading shared state """
        if THREAD_SAFE:
            seq_id = self.shared.get_sequence_id()
        else:
            seq_id = self.shared.sleepy_sequence_id()
        if message == "Echo":
            return f"Echo Reply {seq_id}"
        else:
            return f"Parameter Problem {seq_id}"


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    debugging_config = {'allow_all_attrs': True, 'sync_request_timeout': None}
    echo_svc = rpyc.ThreadedServer(service=SharingService, port=18861, protocol_config=debugging_config)
    echo_svc.start()



from rpyc.core import brine
from rpyc.lib.compat import is_py_3k
import unittest


class BrineTest(unittest.TestCase):
    def test_brine_2(self):
        if is_py_3k:
            exec('''x = (b"he", 7, "llo", 8, (), 900, None, True, Ellipsis, 18.2, 18.2j + 13,
                 slice(1, 2, 3), frozenset([5, 6, 7]), NotImplemented, (1,2))''', globals())
        else:
            exec('''x = ("he", 7, u"llo", 8, (), 900, None, True, Ellipsis, 18.2, 18.2j + 13,
                 slice(1, 2, 3), frozenset([5, 6, 7]), NotImplemented, (1,2))''')
        self.assertTrue(brine.dumpable(x))  # noqa
        y = brine.dump(x)  # noqa
        z = brine.load(y)
        self.assertEqual(x, z)  # noqa


if __name__ == "__main__":
    unittest.main()



from __future__ import with_statement
from rpyc import Service, async_
from rpyc.utils.server import ThreadedServer
from threading import RLock


USERS_DB = {
    "foo": "bar",
    "spam": "bacon",
    "eggs": "viking",
}
broadcast_lock = RLock()
tokens = set()


class UserToken(object):
    def __init__(self, name, callback):
        self.name = name
        self.stale = False
        self.callback = callback
        self.broadcast(f"* Hello {self.name} *")
        tokens.add(self)

    def exposed_say(self, message):
        if self.stale:
            raise ValueError("User token is stale")
        self.broadcast(f"[{self.name}] {message}")

    def exposed_logout(self):
        if self.stale:
            return
        self.stale = True
        self.callback = None
        tokens.discard(self)
        self.broadcast(f"* Goodbye {self.name} *")

    def broadcast(self, text):
        global tokens
        stale = set()
        with broadcast_lock:
            for tok in tokens:
                try:
                    tok.callback(text)
                except Exception:
                    stale.add(tok)
            tokens -= stale


class ChatService(Service):
    def on_connect(self, conn):
        self.token = None

    def on_disconnect(self, conn):
        if self.token:
            self.token.exposed_logout()

    def exposed_login(self, username, password, callback):
        if self.token and not self.token.stale:
            raise ValueError("already logged in")
        if username in USERS_DB and password == USERS_DB[username]:
            self.token = UserToken(username, async_(callback))
            return self.token
        else:
            raise ValueError("wrong username or password")


if __name__ == "__main__":
    t = ThreadedServer(ChatService, port=19912)
    t.start()
